// $Id: datastrg-mem.cpp 356 2015-09-20 09:58:50Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <limits.h>
#include <akamoz-data.h>

namespace akamoz {

DataStorageMemory::DataStorageMemory(darray<BYTE> *pda)
{
	_init();
	mpdabBuf = pda;
}

int DataStorageMemory::setbuf(darray<BYTE> *pda)
{
	if (mpdabBuf != NULL)
		return AKAMOZ_ERR(-1);
	mpdabBuf = pda;
	return 0;
}

int DataStorageMemory::_expand(int nSize)
{
	assert(nSize > 0);
	if (mnPos > INT_MAX - nSize)
		return AKAMOZ_ERR(-1);
	while (mnSize < mnPos + nSize) {
		if (mnSize > INT_MAX / 2) {
			mnSize = INT_MAX;
			break;
		}
		else if (mnSize == 0)
			mnSize = 16;
		else
			mnSize *= 2;
	}
	if (mpdabBuf->reallocate(mnSize) < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

int DataStorageMemory::putbyte(BYTE b)
{
	if (mnPos >= mnSize) {
		if (_expand(1) < 0)
			return AKAMOZ_ERR(-1);
		if (mnPos >= mnSize)
			return AKAMOZ_ERR(-1);
	}
	assert(mnPos < mpdabBuf->size());
	mpdabBuf->at(mnPos++) = b;
	return 0;
}

int DataStorageMemory::write(const BYTE *pb, int nLen)
{
	if (nLen < 0)
		return AKAMOZ_ERR(-1);
	if (nLen == 0)
		return 0;
	if (mnSize < mnPos + nLen) {
		if (_expand(nLen) < 0)
			return AKAMOZ_ERR(-1);
		if (mnSize < mnPos + nLen)
			return AKAMOZ_ERR(-1);
	}
	assert(mnSize >= mnPos + nLen);
	mpdabBuf->setat(mnPos, pb, nLen);
	mnPos += nLen;
	return 0;
}

int DataStorageMemory::flush()
{
	return 0;
}

int DataStorageMemory::cleanup()
{
	_init();
	return 0;
}

} // namespace akamoz
