// $Id: datastrg-stdc.cpp 356 2015-09-20 09:58:50Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <akamoz-data.h>

namespace akamoz {

DataStorageStdc::DataStorageStdc(const char *pfn)
{
	mfp = fopen(pfn, "wb");
}

DataStorageStdc::~DataStorageStdc()
{
	_cleanup();
}

int DataStorageStdc::putbyte(BYTE b)
{
	if (mfp == NULL)
		return AKAMOZ_ERR(-1);
	return fputc(b, mfp) >= 0 ? 0 : AKAMOZ_ERR(-1);
}

int DataStorageStdc::write(const BYTE *pb, int nLen)
{
	if (mfp == NULL)
		return AKAMOZ_ERR(-1);
	while (nLen > 0) {
		int nWritten = fwrite(pb, 1, nLen, mfp);
		if (nWritten <= 0)
			return AKAMOZ_ERR(-1);
		nLen -= nWritten;
		if (nLen < 0)
			return AKAMOZ_ERR(-1);
	}
	return 0;
}

int DataStorageStdc::flush()
{
	if (mfp == NULL)
		return AKAMOZ_ERR(-1);
	fflush(mfp);
	return 0;
}

int DataStorageStdc::cleanup()
{
	_cleanup();
	_init();
	return 0;
}

void DataStorageStdc::_cleanup()
{
	if (mfp != NULL)
		fclose(mfp);
}

} // namespace akamoz
