// $Id: readblockwin32.cpp 353 2015-09-19 16:06:32Z you $
/*
 * Copyright (c) 2008-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <akamoz-data.h>

namespace akamoz2 {

ReadLargeBlockWin32::~ReadLargeBlockWin32()
{
	cleanup();
}

ReadLargeBlockWin32 *ReadLargeBlockWin32::create()
{
	return new(std::nothrow) ReadLargeBlockWin32;
}

void ReadLargeBlockWin32::operator delete(void *p)
{
	::operator delete(p);
}

template <typename TC>
int ReadLargeBlockWin32::setup(const TC *lpszName)
{
	if (isReady())
		return AKAMOZ_ERR(-1);
	mhFile = createFile(lpszName, GENERIC_READ, FILE_SHARE_READ, NULL, 
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (mhFile == INVALID_HANDLE_VALUE)
		return AKAMOZ_ERR(-1);
	mbReady = 1;
	return 0;
}

template int ReadLargeBlockWin32::setup(LPCSTR lpszName);
template int ReadLargeBlockWin32::setup(LPCWSTR lpszName);

int ReadLargeBlockWin32::read(BYTE *pb, int nSize)
{
	if (!isReady())
		return AKAMOZ_ERR(-1);
	if (pb == NULL)
		return AKAMOZ_ERR(-1);
	if (nSize < 0)
		return AKAMOZ_ERR(-1);
	DWORD dwRead = 0;
	if (::ReadFile(mhFile, pb, nSize, &dwRead, NULL) == FALSE)
		return AKAMOZ_ERR(-1);
	return (int)dwRead;
}

int ReadLargeBlockWin32::rewind()
{
	return setpos(0);
}

int ReadLargeBlockWin32::cleanup()
{
	int nRet = 0;
	if ((mhFile != INVALID_HANDLE_VALUE) && (::CloseHandle(mhFile) == FALSE))
		nRet = AKAMOZ_ERR(-1);
	_init();
	return nRet;
}

int ReadLargeBlockWin32::setpos(LLONG nPos)
{
	if (!isReady())
		return AKAMOZ_ERR(-1);
	LARGE_INTEGER liPos;
	liPos.QuadPart = nPos;
	if (::SetFilePointerEx(mhFile, liPos, NULL, FILE_BEGIN) == FALSE)
		return AKAMOZ_ERR(-1);
	return 0;
}

}
