// $Id: readdata.cpp 357 2015-09-20 09:59:38Z you $
/*
 * Copyright (c) 2008-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <akamoz-data.h>

namespace akamoz2 {

ReadDataBase::~ReadDataBase()
{
}

int ReadDataBase::_setup(BYTE *pbBuf, int nBufSize)
{
	if (mprb.isNULL())
		return AKAMOZ_ERR(-1);
	if (pbBuf == AKAMOZ_NULL)
		mdaBuf.reuse(nBufSize);
	else
		mdaBuf.set(pbBuf, nBufSize);
	if (mdaBuf.getmp() == AKAMOZ_NULL)
		return AKAMOZ_ERR(-1);
	return 0;
}

int ReadDataBase::_fill_and_getbyte()
{
	if (!isReady())
		return AKAMOZ_ERR(-1);
	assert(mdaBuf.getmp() != AKAMOZ_NULL);
	assert(mdaBuf.size() > 0);
	mnAvail = _read(mdaBuf.getmp(), mdaBuf.size());
	if (mnAvail == 0) {
		mnPos = 0;
		return -1;	// EOF
	}
	if (mnAvail < 0)
		return AKAMOZ_ERR(-1);
	mnPos = 1;
	return mdaBuf[0];
}

int ReadDataBase::read(BYTE *pb, int n)
{
	if (pb == AKAMOZ_NULL)
		return AKAMOZ_ERR(-1);
	if (n < 0)
		return AKAMOZ_ERR(-1);
	if (!isReady())
		return AKAMOZ_ERR(-1);

	int nRest = mnAvail - mnPos;
	if (nRest < 0)
		return AKAMOZ_ERR(-1);

	assert(mdaBuf.getmp() != AKAMOZ_NULL);
	if (nRest >= n) {
		memcpy(pb, mdaBuf + mnPos, n);
		mnPos += n;
		return n;
	}

	memcpy(pb, mdaBuf + mnPos, nRest);
	int nRead = _read(pb + nRest, n - nRest);
	if (nRead < 0)
		return AKAMOZ_ERR(-1);
	mnPos = mnAvail = nRead;	// used in ReadFileBase<T>::_read
	return nRest + nRead;
}

int ReadDataBase::_read(BYTE *pb, int n)
{
	assert(pb != NULL);
	assert(n >= 0);
	assert(!mprb.isNULL());
	return mprb->read(pb, n);
}

int ReadDataBase::rewind()
{
	if (!isReady())
		return AKAMOZ_ERR(-1);
	assert(!mprb.isNULL());
	if (mprb->rewind() < 0)
		return AKAMOZ_ERR(-1);
	mnPos = mnAvail = 0;
	return 0;
}

int ReadDataBase::cleanup()
{
	int nRet = 0;
	if (!mprb.isNULL() && (mprb->cleanup() < 0))
		nRet = AKAMOZ_ERR(-1);
	mprb.cleanup();
	_init();
	return nRet;
}

} // namespace akamoz2
