// $Id: avisingleif.cpp 353 2015-09-19 16:06:32Z you $
/*
 * Copyright (c) 2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <akamoz-image.h>

#ifndef AKAMOZ_HAVE_WIN32VFW
#define streamtypeVIDEO (akamoz4::LittleEndian::tofcc("vids"))
#define streamtypeAUDIO (akamoz4::LittleEndian::tofcc("auds"))
#define streamtypeMIDI (akamoz4::LittleEndian::tofcc("mids"))
#define streamtypeTEXT (akamoz4::LittleEndian::tofcc("txts"))
#define streamtypeANY	(0)
#endif

namespace akamoz4 {

AVISingleStreamIF::~AVISingleStreamIF()
{
}

void AVISingleStreamIF::cleanup() throw()
{
	_init();
}

int AVISingleStreamIF::getNumStreams(DWORD fccid) const
{
	if (fccid == 0)
		return getNumStreams();
	if (mstrmap.count(fccid) < 1)
		return 0;
	return mstrmap.find(fccid)->second.size();
}

int AVISingleStreamIF::selectVideoStream(int n)
{
	return selectStream(n, streamtypeVIDEO);
}

int AVISingleStreamIF::selectAudioStream(int n)
{
	return selectStream(n, streamtypeAUDIO);
}

int AVISingleStreamIF::selectMidiStream(int n)
{
	return selectStream(n, streamtypeMIDI);
}

int AVISingleStreamIF::selectTextStream(int n)
{
	return selectStream(n, streamtypeTEXT);
}

int AVISingleStreamIF::getNumVideoStreams() const
{
	return getNumStreams(streamtypeVIDEO);
}

int AVISingleStreamIF::getNumAudioStreams() const
{
	return getNumStreams(streamtypeAUDIO);
}

int AVISingleStreamIF::getNumMidiStreams() const
{
	return getNumStreams(streamtypeMIDI);
}

int AVISingleStreamIF::getNumTextStreams() const
{
	return getNumStreams(streamtypeTEXT);
}

// return information from 'strh' chunk.
int AVISingleStreamIF::getStreamSummary(AVIStreamSummary &info, int nStr) const
{
	if (nStr < 0) {
		nStr = mnStreamIndex;
		if (nStr < 0)
			return AKAMOZ_ERR(-1);
	}
	if (nStr >= getNumStreams())
		return AKAMOZ_ERR(-1);
	info = mdaStreamInfo[nStr];
	return 0;
}

// return information from 'strf' chunk
const DataBuffer *AVISingleStreamIF::getStreamDetail(int nStr) const
{
	if (nStr < 0) {
		nStr = mnStreamIndex;
		if (nStr < 0)
			return AKAMOZ_ERRPTR(const DataBuffer *);
	}
	if (nStr >= mnNumStreams)
		return AKAMOZ_ERRPTR(const DataBuffer *);
	if (!mdaStreamInfo.isReady())
		return AKAMOZ_ERRPTR(const DataBuffer *);
	return &mdaStreamInfo[nStr].dbFormat;
}

const DataBuffer *AVISingleStreamIF::getStreamSupplement(int) const
{
	// read 'strd' chunk at setup, and return it here.
	// it shoud be used for ICSetState later.
	return AKAMOZ_ERRPTR(const DataBuffer *);	// not implemented
}

} // namespace akamoz4
