// $Id: avistream2.cpp 353 2015-09-19 16:06:32Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <akamoz-image.h>
#include <ctype.h>
#include <iostream>

namespace akamoz2 {

AVIStream::AVIStream() throw()
{
	akamoz::gAVIiniter.init();	// share with the first generation of akamoz
	_init();
}

AVIStream::~AVIStream() throw()
{
}

int AVIStream::readAVIMainHeader(ReadLargeFileGeneric &rf)
{
	DWORD dw;
	if (rf.forward(12) < 0)
		return AKAMOZ_ERR(-1);
	if (rf.getdwordLE(dw) < 0)	// dwFlags;
		return AKAMOZ_ERR(-1);
	if (dw & 0x20)	// AVIF_MUSTUSEINDEX
		return AKAMOZ_ERR(-1);	// index is not supported.
	if (rf.forward(8) < 0)
		return AKAMOZ_ERR(-1);
	if (rf.getdwordLE(mnNumStreams) < 0)
		return AKAMOZ_ERR(-1);
	if (mnNumStreams < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

int AVIStream::readAVIStreamHeader(ReadLargeFileGeneric &rfStrl, AVIStreamDetail &info)
{
	ReadLargeIFF iff;
	if (iff.setup(&rfStrl) < 0)
		return AKAMOZ_ERR(-1);
	if (iff.findck("strh") < 0)
		return AKAMOZ_ERR(-1);
	ReadLargeFileGeneric &rf = iff.body();

	DWORD dw;
	if (rf.getdwordLE(dw) < 0)
		return AKAMOZ_ERR(-1);
	if (cmpfcc(dw, "vids") == 0)
		info.nType = aviVideo;
	else if (cmpfcc(dw, "auds") == 0)
		info.nType = aviAudio;
	else if (cmpfcc(dw, "mids") == 0)
		info.nType = aviMidi;
	else if (cmpfcc(dw, "txts") == 0)
		info.nType = aviText;
	else
		info.nType = aviUnknown;
	if (rf.getdwordLE(info.dwHandler) < 0)
		return AKAMOZ_ERR(-1);
	if (rf.forward(12) < 0)
		return AKAMOZ_ERR(-1);
	if (rf.getdwordLE(info.dwRateDenom) < 0)
		return AKAMOZ_ERR(-1);
	if (rf.getdwordLE(info.dwRateNum) < 0)
		return AKAMOZ_ERR(-1);
	if (rf.forward(4) < 0)
		return AKAMOZ_ERR(-1);
	if (rf.getdwordLE(info.dwLength) < 0)
		return AKAMOZ_ERR(-1);
	if (iff.nextck() < 0)
		return AKAMOZ_ERR(-1);

	if (iff.findck("strf") < 0)
		return AKAMOZ_ERR(-1);
	int nSize = iff.getcksize();
	if (nSize < 0)
		return AKAMOZ_ERR(-1);
	if (info.dbFormat.reuse(nSize) < 0)
		return AKAMOZ_ERR(-1);
	if (iff.body().read(info.dbFormat.getmp(), nSize) != nSize)
		return AKAMOZ_ERR(-1);

	return 0;
}

template <typename TC>
int AVIStream::setup(const TC *pszfn) throw()
{
	if (mrf.setup(pszfn) < 0)
		return AKAMOZ_ERR(-1);
	if (miffRIFF.setup(&mrf) < 0)
		return AKAMOZ_ERR(-1);
	if (!miffRIFF.ckidis("RIFF"))
		return AKAMOZ_ERR(-1);
	DWORD dwID;
	if (miffRIFF.body().getdwordLE(dwID) < 0)
		return AKAMOZ_ERR(-1);
	if (cmpfcc(dwID, "AVI ") != 0)
		return AKAMOZ_ERR(-1);
	if (miffAVI.setup(&miffRIFF.body()) < 0)
		return AKAMOZ_ERR(-1);
	if (miffAVI.findlist("hdrl") < 0)
		return AKAMOZ_ERR(-1);

	ReadLargeIFF iffHdrl;
	if (iffHdrl.setup(&miffAVI.body()) < 0)
		return AKAMOZ_ERR(-1);
	if (iffHdrl.findck("avih") < 0)
		return AKAMOZ_ERR(-1);
	if (readAVIMainHeader(iffHdrl.body()) < 0)
		return AKAMOZ_ERR(-1);
	if (mdaStreamInfo.allocate(getNumStreams()) < 0)
		return AKAMOZ_ERR(-1);
	for (int i = 0; i < getNumStreams(); i++) {
		if (iffHdrl.findlist("strl") < 0)
			return AKAMOZ_ERR(-1);
		if (readAVIStreamHeader(iffHdrl.body(), mdaStreamInfo[i]) < 0)
			return AKAMOZ_ERR(-1);
		if (iffHdrl.nextck() < 0)
			return AKAMOZ_ERR(-1);
	}
	if (iffHdrl.cleanup() < 0)
		return AKAMOZ_ERR(-1);
	mnLoopStat = 1;
	if (nextFrame() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template int AVIStream::setup(LPCSTR pszfn) throw();
template int AVIStream::setup(LPCWSTR pszfn) throw();

int AVIStream::getStreamSummary(int nStr, AVIStreamSummary &info)
{
	if (nStr < 0)
		return AKAMOZ_ERR(-1);
	if (nStr >= getNumStreams())
		return AKAMOZ_ERR(-1);
	info = mdaStreamInfo[nStr];
	return 0;
}

const DataBuffer *AVIStream::getStreamDetail(int nStr)
{
	if (nStr < 0)
		return AKAMOZ_ERRPTR(const DataBuffer *);
	if (nStr >= mnNumStreams)
		return AKAMOZ_ERRPTR(const DataBuffer *);
	if (!mdaStreamInfo.isReady())
		return AKAMOZ_ERRPTR(const DataBuffer *);
	return &mdaStreamInfo[nStr].dbFormat;
}

int AVIStream::getVideoStream(int nStr)
{
	if (nStr < 0)
		return AKAMOZ_ERR(-1);
	int nVideoStream = 0;
	for (int i = 0; i < mnNumStreams; i++){
		if (mdaStreamInfo[i].nType == aviVideo) {
			if (nVideoStream == nStr)
				return i;
			nVideoStream++;
		}
	}
	return AKAMOZ_ERR(-1);
}

int AVIStream::getNextStrnum()
{
	const char *pid = miffMovi.getckid();
	if (!isdigit(pid[0]) || !isdigit(pid[1]))
		return AKAMOZ_ERR(-1);
	// 16iA10iH
	return (pid[0] - '0') * 10 + pid[1] - '0';
}

#define MARK	(std::cerr << "avistr2:" << __LINE__ << " ")

int AVIStream::nextFrame()
{
	int bEOF = 0;
	while (!bEOF) {
		switch (mnLoopStat) {
		  case 0:
			if (miffRIFF.findck("RIFF") < 0) {
				bEOF = 1;
				break;	// EOF, maybe.
			}
			DWORD dw;
			if (miffRIFF.body().getdwordLE(dw) < 0)
				return AKAMOZ_ERR(-1);
			if (cmpfcc(dw, "AVIX") != 0) {
				mnLoopStat = 5;
				break;
			}
			if (miffAVI.setup(&miffRIFF.body()) < 0)
				return AKAMOZ_ERR(-1);
		  case 1:
			if (miffAVI.findlist("movi") < 0) {
				mnLoopStat = 2;
				break;
			}
			if (miffMovi.setup(&miffAVI.body()) < 0)
				return AKAMOZ_ERR(-1);
		  case 3: {
				const char *id = miffMovi.getckid();
				if (isdigit(id[0]) && isdigit(id[1])) {
					mnLoopStat = 6;
					return 1;
				}
			}
			// is break missing or not?
		  case 6:
			if (miffMovi.nextck() >= 0) {
				mnLoopStat = 3;
				break;
			}
		  case 4:
			if (miffMovi.cleanup() < 0)
				return AKAMOZ_ERR(-1);
			if (miffAVI.nextck() >= 0) {
				mnLoopStat = 1;
				break;
			}
		  case 2:
			if (miffAVI.cleanup() < 0)
				return AKAMOZ_ERR(-1);
		  case 5:
			if (miffRIFF.nextck() < 0) {
				bEOF = 1;
				break;	// EOF, maybe.
			}
			mnLoopStat = 0;
			break;
		}
	}
	return 0;
}

int AVIStream::getFrame(DataBuffer *pdb)
{
	if (pdb == NULL)
		return AKAMOZ_ERR(-1);
	int nStream = getNextStrnum();
	if (nStream < 0)
		return AKAMOZ_ERR(-1);
	LLONG nSize = miffMovi.getcksize();
	if (nSize < 0)
		return AKAMOZ_ERR(-1);
	if (nSize > 0x7fffffff)
		return AKAMOZ_ERR(-1);
	if (pdb->reuse(nSize) < 0)
		return AKAMOZ_ERR(-1);
	if (pdb->getmp() == NULL)
		return AKAMOZ_ERR(-1);
	if (miffMovi.body().read(pdb->getmp(), nSize) != nSize)
		return AKAMOZ_ERR(-1);
	if (nextFrame() < 0)
		return AKAMOZ_ERR(-1);
	return nStream;
}

} // namespace akamoz2
