// $Id: imagebuffer.cpp 357 2015-09-20 09:59:38Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "akamoz-image.h"

namespace akamoz {

ImageBuffer::ImageBuffer()
{
	_init();
}

ImageBuffer::~ImageBuffer()
{
}

int ImageBuffer::cleanup()
{
	if (mdabBuf.cleanup() < 0)
		return AKAMOZ_ERR(-1);
	_init();
	return 0;
}

int ImageBuffer::setProperty(int nWidth, int nHeight, int nGuard, int nAlign)
{
	mnWidth = mnHeight = mnImageSize = -1;
	if (nWidth < 1)
		return AKAMOZ_ERR(-1);
	if (nHeight < 1)
		return AKAMOZ_ERR(-1);
	if (nGuard >= 0)
		mnGuardBytes = nGuard;
	if (nAlign > 0)
		mnAlign = nAlign;
	int nLineSize = _calcLineSize(nWidth);
	if (nLineSize < 0)
		return AKAMOZ_ERR(-1);
	int nImageSize = nLineSize * nHeight;
	if (nImageSize / nHeight != nLineSize)
		return AKAMOZ_ERR(-1);
	if (mdabBuf.reuse(nImageSize + mnGuardBytes * 2 + mnAlign - 1) < 0)
		return AKAMOZ_ERR(-1);
	mnWidth = nWidth;
	mnHeight = nHeight;
	mnLineSize = nLineSize;
	mnImageSize = nImageSize;
	PTRINT ptr = (PTRINT)(mdabBuf + mnGuardBytes + mnAlign - 1) & (-(PTRINT)mnAlign);
	mpbBuf = (BYTE *)ptr;

	return 0;
}

int ImageBuffer::_calcLineSize(int nWidth)
{
	int nLineSize = nWidth * 3;
	if (nLineSize / 3 != nWidth)
		return AKAMOZ_ERR(-1);
	nLineSize += 3;
	if (nLineSize < 0)
		return AKAMOZ_ERR(-1);
	nLineSize /= 4;
	return nLineSize * 4;
}

};
