// $Id: imagebuffer2.cpp 357 2015-09-20 09:59:38Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <akamoz-image.h>

namespace akamoz2 {

ImageBuffer::~ImageBuffer()
{
}

int ImageBuffer::setup(int x, int y, int bpp, BYTE *pbBuf) throw()
{
	if (setup_base(x, y, bpp) < 0)
		return AKAMOZ_ERR(-1);
	if (pbBuf == NULL) {
		if (mdb.reuse(mnImageSize) < 0)
			return AKAMOZ_ERR(-1);
	}
	else {
		if (mdb.set(pbBuf, mnImageSize) < 0)
			return AKAMOZ_ERR(-1);
	}
	if (_postsetup() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

int ImageBuffer::setup(int x, int y, int bpp, const BYTE *pbBuf) throw()
{
	if (setup_base(x, y, bpp) < 0)
		return AKAMOZ_ERR(-1);
	if (pbBuf == NULL)
		return AKAMOZ_ERR(-1);
	if (mdb.set(pbBuf, mnImageSize) < 0)
		return AKAMOZ_ERR(-1);
	if (_postsetup() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

int ImageBuffer::setup_base(int x, int y, int bpp) throw()
{
	if (x <= 0)
		return AKAMOZ_ERR(-1);
	if (y <= 0)
		return AKAMOZ_ERR(-1);
	if (bpp <= 0)
		return AKAMOZ_ERR(-1);
	mnWidth = x;
	mnHeight = y;
	msbBPP = bpp;
	if (_setup() < 0)
		return AKAMOZ_ERR(-1);
	if (mnImageSize <= 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

int ImageBuffer::_setup()
{
	if (_setPacking() < 0)
		return AKAMOZ_ERR(-1);
	if (_calcLineSize() < 0)
		return AKAMOZ_ERR(-1);
	if (_calcImageSize() < 0)
		return AKAMOZ_ERR(-1);
	if (_setOptions() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

int ImageBuffer::_setPacking()
{
	switch (msbBPP) {
	  case 1: case 2: case 4:
		msbPackSize = 1;
		msbPixPacked = 8 / msbBPP;
		break;
	  case 8: case 16: case 24: case 32:
		msbPackSize = msbBPP / 8;
		msbPixPacked = 1;
		break;
	  case 6: case 12:
		msbPackSize = 3;
		msbPixPacked = 24 / msbBPP;
		break;
	  default:
		return AKAMOZ_ERR(-1);
	}
	return 0;
}

int ImageBuffer::_calcLineSize()
{
	mnLineSize = mnWidth * msbBPP;
	if (mnLineSize / msbBPP != mnWidth)
		return AKAMOZ_ERR(-1);
	mnLineSize += 31;
	if (mnLineSize < 0)
		return AKAMOZ_ERR(-1);
	mnLineSize /= 32;
	mnLineSize *= 4;
	return 0;
}

int ImageBuffer::_calcImageSize()
{
	if (_calcLineSize() < 0)
		return AKAMOZ_ERR(-1);
	mnImageSize = mnLineSize * mnHeight;
	if (mnImageSize / mnHeight != mnLineSize)
		return AKAMOZ_ERR(-1);
	return 0;
}

int ImageBuffer::_setOptions()
{
	return 0;
}

int ImageBuffer::_postsetup()
{
	return 0;
}

}
