// $Id: imgbuf-yv12.cpp 358 2015-09-20 10:00:06Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <akamoz-image.h>

namespace akamoz2 {

void ImageBufferYV12Imp::convLuma(const ImageBuffer *psrc, ImageBuffer *pdst)
{
	ImageItorConst itrSrc(psrc);
	ImageItor itrDst(pdst);
	for (int y = 0; y < psrc->getHeight(); y++) {
		const BYTE *pbSrc = *itrSrc++;
		BYTE *pbDst = *itrDst++;
		for (int x = 0; x < psrc->getWidth(); x++) {
			*pbDst = *pbSrc;
			pbSrc += psrc->getPackSize();
			pbDst++;
		}
	}
}

void ImageBufferYV12Imp::convChroma2P(const ImageBuffer *psrc, ImageBuffer *pdst)
{
	ImageItorConst itrSrc(psrc);
	ImageItor itrDst(pdst);
	for (int y = 0; y < psrc->getHeight(); y += 2) {
		const BYTE *pbSrc0 = *itrSrc++;
		const BYTE *pbSrc1 = *itrSrc++;
		BYTE *pbDst = *itrDst++;
		for (int x = 0; x < psrc->getWidth(); x++) {
			*pbDst++ = (*pbSrc0 + *pbSrc1 + 1) >> 1;
			pbSrc0 += psrc->getPackSize();
			pbSrc1 += psrc->getPackSize();
		}
	}
}

void ImageBufferYV12Imp::convChroma2I(const ImageBuffer *psrc, ImageBuffer *pdst)
{
	ImageItorConst itrSrc(psrc);
	ImageItor itrDst(pdst);
	for (int y = 0; y < psrc->getHeight(); y += 4) {
		const BYTE *pbSrc0 = *itrSrc++;
		const BYTE *pbSrc1 = *itrSrc++;
		const BYTE *pbSrc2 = *itrSrc++;
		const BYTE *pbSrc3 = *itrSrc++;
		BYTE *pbDst0 = *itrDst++;
		BYTE *pbDst1 = *itrDst++;
		for (int x = 0; x < psrc->getWidth(); x++) {
			*pbDst0++ = (*pbSrc0 * 3 + *pbSrc2 + 2) >> 2;
			*pbDst1++ = (*pbSrc1 + *pbSrc3 * 3 + 2) >> 2;
			pbSrc0 += psrc->getPackSize();
			pbSrc1 += psrc->getPackSize();
			pbSrc2 += psrc->getPackSize();
			pbSrc3 += psrc->getPackSize();
		}
	}
}

int ImageBufferYV12Imp::setup(const ImageBufferYUY2 *pimg, int bInterlaced)
{
	if (pimg == NULL)
		return AKAMOZ_ERR(-1);
	if (ImageBuffer::setup(pimg->getWidth(), pimg->getHeight(), 12) < 0)
		return AKAMOZ_ERR(-1);
	convLuma(&pimg->Y(), &Y());
	if (bInterlaced) {
		convChroma2I(&pimg->U(), &U());
		convChroma2I(&pimg->V(), &V());
	}
	else {
		convChroma2P(&pimg->U(), &U());
		convChroma2P(&pimg->V(), &V());
	}
	return 0;
}

template <typename T>
int ImageBufferYV12Imp::_setupPlanes(T *p)
{
	if (mimgY.setup(getWidth(), getHeight(), 8, p) < 0)
		return AKAMOZ_ERR(-1);
	if (mimgY.setPixSpan(1) < 0)
		return AKAMOZ_ERR(-1);
	p += mimgY.getImageSize();
	if (mimgV.setup(getWidth() / 2, getHeight() / 2, 8, p) < 0)
		return AKAMOZ_ERR(-1);
	if (mimgV.setPixSpan(1) < 0)
		return AKAMOZ_ERR(-1);
	p += mimgV.getImageSize();
	if (mimgU.setup(getWidth() / 2, getHeight() / 2, 8, p) < 0)
		return AKAMOZ_ERR(-1);
	if (mimgU.setPixSpan(1) < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template int ImageBufferYV12Imp::_setupPlanes(BYTE *);
template int ImageBufferYV12Imp::_setupPlanes(const BYTE *);
template class ImageBufferYUVImp<ImageBufferYV12Imp>;


}
