// $Id: readbmp.cpp 358 2015-09-20 10:00:06Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <windows.h>
#include "akamoz-image.h"

namespace akamoz {

int readBitmapFile(ImageBuffer *pimg, const char *fn)
{
	FILE *fp = fopen(fn, "rb");
	if (fp == NULL)
		return AKAMOZ_ERR(-1);
	BITMAPFILEHEADER bmfh;
	if (fread(&bmfh, sizeof(bmfh), 1, fp) != 1)
		return AKAMOZ_ERR(-1);
	if (bmfh.bfType != 0x4d42)
		return AKAMOZ_ERR(-1);

	BITMAPINFOHEADER bmih;
	if (fread(&bmih, sizeof(bmih), 1, fp) != 1)
		return AKAMOZ_ERR(-1);
	if (bmih.biSize != sizeof(bmih))
		return AKAMOZ_ERR(-1);
	if (bmih.biCompression != BI_RGB)
		return AKAMOZ_ERR(-1);
	if (bmih.biBitCount != 24)
		return AKAMOZ_ERR(-1);

	int nWidth = bmih.biWidth;
	if (nWidth < 0)
		return AKAMOZ_ERR(-1);
	int nHeight = bmih.biHeight;
	if (nHeight < 0)
		return AKAMOZ_ERR(-1);

	if (pimg->setProperty(nWidth, nHeight) < 0)
		return AKAMOZ_ERR(-1);
	BYTE *pbImg = pimg->getPixelBuffer();
	if (pbImg == NULL)
		return AKAMOZ_ERR(-1);

	int nSize = pimg->getImageSize();
	if (nSize < 0)
		return AKAMOZ_ERR(-1);
	if (fseek(fp, bmfh.bfOffBits, SEEK_SET) != 0)
		return AKAMOZ_ERR(-1);
	if (fread(pbImg, 1, nSize, fp) != (size_t)nSize)
		return AKAMOZ_ERR(-1);
	if (fclose(fp) != 0)
		return AKAMOZ_ERR(-1);

	return 0;
}

}
