// $Id: vcmdecode.cpp 358 2015-09-20 10:00:06Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <akamoz-image.h>

namespace akamoz2 {

vcmDecode::~vcmDecode()
{
	cleanup();
}

int vcmDecode::setup(const AVIStreamSummary &aviinfo, const BITMAPINFOHEADER *pbmih, DWORD fccTarget, int bpp)
{
	if (pbmih == NULL)
		return AKAMOZ_ERR(-1);
	if (pbmih->biSize >= 0x100000)
		return AKAMOZ_ERR(-1);
	if (pbmih->biSize < sizeof(BITMAPINFOHEADER))
		return AKAMOZ_ERR(-1);
	if (mdabmihSrc.allocate((int)pbmih->biSize) < 0)
		return AKAMOZ_ERR(-1);
	memcpy((BYTE *)mdabmihSrc, pbmih, pbmih->biSize);

	BITMAPINFOHEADER *pbmihTarget = NULL;
	if (fccTarget != 0xffffffff) {
		pbmihTarget = &mbmihDst;
		mbmihDst = *pbmih;
		mbmihDst.biSize = sizeof(mbmihDst);
		mbmihDst.biCompression = fccTarget;
		mbmihDst.biBitCount = bpp;
	}
	mhic = ICDecompressOpen(ICTYPE_VIDEO, aviinfo.dwHandler, 
		(BITMAPINFOHEADER *)(BYTE *)mdabmihSrc, pbmihTarget);
	if (mhic == NULL)
		return AKAMOZ_ERR(-1);
	if (fccTarget == 0xffffffff) {
		mbmihDst.biSize = sizeof(mbmihDst);
		if (ICDecompressGetFormat(mhic, pbmih, &mbmihDst) != ICERR_OK)
			return AKAMOZ_ERR(-1);
	}
	if (ICDecompressBegin(mhic, pbmih, &mbmihDst) != ICERR_OK)
		return AKAMOZ_ERR(-1);
	mbBegun = 1;
	return 0;
}

int vcmDecode::decode(DataBuffer *pdbSrc, ImageBuffer *pimgDst)
{
	if (pdbSrc == NULL) return AKAMOZ_ERR(-1);
	if (pimgDst == NULL) return AKAMOZ_ERR(-1);

	if (mhic == NULL) return AKAMOZ_ERR(-1);

	BITMAPINFOHEADER *pbmihSrc = (BITMAPINFOHEADER *)(BYTE *)mdabmihSrc;
	int nLen = pdbSrc->size();
	if (nLen < 0) return AKAMOZ_ERR(-1);
	pbmihSrc->biSizeImage = nLen;

	BYTE *pbSrc = pdbSrc->getmp();
	if (pbSrc == NULL) return AKAMOZ_ERR(-1);

	BYTE *pbDst = pimgDst->getmp();
	int nImageSize = pimgDst->getImageSize();
	if ((pbDst == NULL) || (nImageSize <= 0)) {
		if (pimgDst->setup(mbmihDst.biWidth, mbmihDst.biHeight, mbmihDst.biBitCount) < 0)
			return AKAMOZ_ERR(-1);
		pbDst = pimgDst->getmp();
		if (pbDst == NULL)
			return AKAMOZ_ERR(-1);
		nImageSize = pimgDst->getImageSize();
		if (nImageSize <= 0)
			return AKAMOZ_ERR(-1);
	}
	mbmihDst.biSizeImage = nImageSize;

	if (ICDecompress(mhic, 0, pbmihSrc, pbSrc, &mbmihDst, pbDst) != ICERR_OK)
		return AKAMOZ_ERR(-1);

	return 0;
}


int vcmDecode::cleanup()
{
	int nRet = 0;
	if (mbBegun) {
		if (ICDecompressEnd(mhic) != ICERR_OK)
			nRet = AKAMOZ_ERR(-1);
	}
	if ((mhic != NULL) && (ICClose(mhic) != ICERR_OK))
		nRet = AKAMOZ_ERR(-1);
	if (mdabmihSrc.cleanup() < 0)
		nRet = AKAMOZ_ERR(-1);
	_init();
	return nRet;
}

int vcmDecode::getInfo(ICINFO &info)
{
	if (mhic == NULL)
		return AKAMOZ_ERR(-1);
	int len = ICGetInfo(mhic, &info, sizeof(info));
	if (len == 0)
		return AKAMOZ_ERR(-1);
	return len;
}

}
