// $Id: writebmp.cpp 359 2015-09-20 10:00:28Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <io.h>
#include <fcntl.h>
#include <akamoz-image.h>

namespace akamoz {

int writeBitmapFile(const ImageBuffer *pimg, FILE *fp)
{
	if (fp == NULL)
		return AKAMOZ_ERR(-1);
#ifdef AKAMOZ_WIN32
	setmode(fileno(fp), O_BINARY);
#endif
	BITMAPFILEHEADER bmfh;
	BITMAPINFOHEADER bmih;
	int nImageSize = pimg->getImageSize();
	if (nImageSize < 0)
		return AKAMOZ_ERR(-1);

	bmfh.bfType = 0x4d42;
	bmfh.bfSize = sizeof(bmfh) + sizeof(bmih) + nImageSize;
	bmfh.bfReserved1 = bmfh.bfReserved2 = 0;
	bmfh.bfOffBits = sizeof(bmfh) + sizeof(bmih);

	bmih.biSize = sizeof(bmih);
	bmih.biWidth = pimg->getWidth();
	bmih.biHeight = pimg->getHeight();
	bmih.biPlanes = 1;
	bmih.biBitCount = 24;
	bmih.biCompression = BI_RGB;
	bmih.biSizeImage = nImageSize;
	bmih.biXPelsPerMeter = 1000;
	bmih.biYPelsPerMeter = 1000;
	bmih.biClrUsed = 0;
	bmih.biClrImportant = 0;

	const BYTE *pbImg = pimg->getPixelBuffer();
	if (pbImg == NULL)
		return AKAMOZ_ERR(-1);

	if (fwrite(&bmfh, sizeof(bmfh), 1, fp) != 1)
		return AKAMOZ_ERR(-1);
	if (fwrite(&bmih, sizeof(bmih), 1, fp) != 1)
		return AKAMOZ_ERR(-1);
	if (fwrite(pbImg, 1, nImageSize, fp) != (size_t)nImageSize)
		return AKAMOZ_ERR(-1);

	return 0;
}

int writeBitmapFile(const ImageBuffer *pimg, const char *fn)
{
	if (pimg == NULL)
		return AKAMOZ_ERR(-1);
	if (fn == NULL)
		return AKAMOZ_ERR(-1);

	FILE *fp = fopen(fn, "wb");
	if (writeBitmapFile(pimg, fp) < 0)
		return AKAMOZ_ERR(-1);
	if (fclose(fp) != 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

};
