// $Id: akamoz-avistream3.t 353 2015-09-19 16:06:32Z you $
/*
 * Copyright (c) 2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __AVISTREAM3_T_INCLUDED
#define __AVISTREAM3_T_INCLUDED

#include <akamoz-image.h>
#include <ctype.h>
#include <iostream>
#include <map>
#include <akamoz-iomanip.h>

namespace akamoz4 {

template <typename TRF, typename TEND>
AVISingleStreamBase<TRF, TEND>::AVISingleStreamBase() throw()
{
	akamoz::gAVIiniter.init();	// share with the first generation of akamoz
	_init();
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::readAVIMainHeader(TChunkReader &ckrd)
{
	DWORD dw;
	if (ckrd->forward(12) < 0)
		return AKAMOZ_ERR(-1);
	if (ckrd.getdword(dw) < 1)	// dwFlags;
		return AKAMOZ_ERR(-1);
	if (dw & 0x20)	// AVIF_MUSTUSEINDEX
		return AKAMOZ_ERR(-1);	// index is not supported.
	if (ckrd.getdword(mdwTotalFrames) < 1)
		return AKAMOZ_ERR(-1);
	if (ckrd->forward(4) < 0)
		return AKAMOZ_ERR(-1);
	if (ckrd.getdword(mnNumStreams) < 1)
		return AKAMOZ_ERR(-1);
	if (mnNumStreams < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::readAVIStreamHeader(TChunkReader &ck, AVIStreamDetail &info)
{
	TChunkReader subck;
	if (ck.findck(subck, "strh") < 1)
		return AKAMOZ_ERR(-1);

	if (subck.getdword(info.fccType) < 1)
		return AKAMOZ_ERR(-1);
	if (fcceq(info.fccType, "vids"))
		info.nType = aviVideo;
	else if (fcceq(info.fccType, "auds"))
		info.nType = aviAudio;
	else if (fcceq(info.fccType, "mids"))
		info.nType = aviMidi;
	else if (fcceq(info.fccType, "txts"))
		info.nType = aviText;
	else
		info.nType = aviUnknown;
	if (subck.getdword(info.dwHandler) < 1)
		return AKAMOZ_ERR(-1);
	if (subck->forward(12) < 0)
		return AKAMOZ_ERR(-1);
	if (subck.getdword(info.dwRateDenom) < 1)
		return AKAMOZ_ERR(-1);
	if (subck.getdword(info.dwRateNum) < 1)
		return AKAMOZ_ERR(-1);
	if (subck->forward(4) < 0)
		return AKAMOZ_ERR(-1);
	if (subck.getdword(info.dwLength) < 1)
		return AKAMOZ_ERR(-1);
	if (subck.cleanup() < 0)
		return AKAMOZ_ERR(-1);

	if (ck.findck(subck, "strf") < 1)
		return AKAMOZ_ERR(-1);
	int nSize = subck.getcksize();
	if (nSize < 0)
		return AKAMOZ_ERR(-1);
	if (info.dbFormat.reuse(nSize) < 0)
		return AKAMOZ_ERR(-1);
	if (subck->read(info.dbFormat.getmp(), nSize) != nSize)
		return AKAMOZ_ERR(-1);
	if (subck.cleanup() < 0)
		return AKAMOZ_ERR(-1);

	return 0;
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::setupAVIFile(TReadFile *prf) throw()
{
	if (mckRIFF.setup(prf) < 0)
		return AKAMOZ_ERR(-1);
	if (!mckRIFF.ckidis("RIFF"))
		return AKAMOZ_ERR(-1);
	DWORD dwID;
	if (mckRIFF.getfcc(dwID) < 1)
		return AKAMOZ_ERR(-1);
	if (!fcceq(dwID, "AVI "))
		return AKAMOZ_ERR(-1);

	TChunkReader ckHeader;
	if (mckRIFF.findlist(ckHeader, "hdrl") < 1)
		return AKAMOZ_ERR(-1);
	TChunkReader ckSubHeader;
	if (ckHeader.findck(ckSubHeader, "avih") < 1)
		return AKAMOZ_ERR(-1);
	if (readAVIMainHeader(ckSubHeader) < 0)
		return AKAMOZ_ERR(-1);
	if (ckSubHeader.cleanup() < 0)
		return AKAMOZ_ERR(-1);
	if (mdaStreamInfo.allocate(mnNumStreams) < 0)
		return AKAMOZ_ERR(-1);
	for (int i = 0; i < mnNumStreams; i++) {
		if (ckHeader.findlist(ckSubHeader, "strl") < 1)
			return AKAMOZ_ERR(-1);
		if (readAVIStreamHeader(ckSubHeader, mdaStreamInfo[i]) < 0)
			return AKAMOZ_ERR(-1);
		mstrmap[mdaStreamInfo[i].fccType].push_back(i);
		if (ckSubHeader.cleanup() < 0)
			return AKAMOZ_ERR(-1);
	}
	// optional chunks
	while (!ckHeader.isckend()) {
		if (ckSubHeader.setup(&mrf) < 1)
			return AKAMOZ_ERR(-1);
		if (ckSubHeader.ckidis("LIST")) {
			DWORD dwid;
			if (ckSubHeader.getdword(dwid) < 1)
				return AKAMOZ_ERR(-1);
			if (fcceq(dwid, "odml")) {
				TChunkReader ck;
				while (!ckSubHeader.isckend()) {
					if (ck.setup(&mrf) < 1)
						return AKAMOZ_ERR(-1);
					if (ck.ckidis("dmlh")) {
						if (ck.getdword(mdwTotalFrames) < 1)
							return AKAMOZ_ERR(-1);
					}
					if (ck.cleanup() < 0)
						return AKAMOZ_ERR(-1);
				}
			}
		}
		if (ckSubHeader.cleanup() < 0)
			return AKAMOZ_ERR(-1);
	}
	if (ckHeader.cleanup() < 0)
		return AKAMOZ_ERR(-1);
	if (mckRIFF.findlist(mckMovi, "movi") < 1)
		return AKAMOZ_ERR(-1);
	return 0;
}

template <typename TRF, typename TEND>
template <typename TC>
int AVISingleStreamBase<TRF, TEND>::setup(const TC *pszfn)
{
	if (mrf.setup(pszfn) < 0)
		return AKAMOZ_ERR(-1);
	if (setupAVIFile(&mrf) < 0) {
		mrf.cleanup();
		return AKAMOZ_ERR(-1);
	}
	return 0;
}

template <typename TRF, typename TEND>
void AVISingleStreamBase<TRF, TEND>::cleanup() throw()
{
	mckRec.cleanup();
	mckMovi.cleanup();
	mckRIFF.cleanup();
	mrf.cleanup();
	mdaStreamInfo.cleanup();
	mstrmap.clear();
	AVISingleStreamIF::cleanup();
	_init();
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::selectStream(int n, DWORD fccid)
{
	if (n < 0)
		return AKAMOZ_ERR(-1);
	mnStreamIndex = -1;
	if (fccid == 0) {
		if (n >= mnNumStreams)
			return AKAMOZ_ERR(-1);
		mnStreamIndex = n;
	}
	else {
		if (mstrmap.count(fccid) < 1)
			return AKAMOZ_ERR(-1);
		if ((UINT)n >= mstrmap[fccid].size())
			return AKAMOZ_ERR(-1);
		mnStreamIndex = mstrmap[fccid][n];
	}
	if (mnStreamIndex >= 100) {
		mnStreamIndex = -1;
		return AKAMOZ_ERR(-1);
	}
	mbFccPrefix[0] = (mnStreamIndex / 10) + '0';
	mbFccPrefix[1] = (mnStreamIndex % 10) + '0';
	return 0;
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::checkStreamPrefix(const TChunkReader &ck)
{
	BYTE b[4];
	TEndian::fromfcc(b, ck.ckid());
	if ((b[0] == mbFccPrefix[0]) && (b[1] == mbFccPrefix[1]))
		return 1;
	return 0;
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::readdata(
	DataBuffer *pdb, const TChunkReader &ck
) {
	int nRead = ck.getcksize();
	if (pdb->reuse(nRead) < 0)
		return AKAMOZ_ERR(-1);
	if (mrf.read(pdb->getmp(), nRead) != nRead)
		return AKAMOZ_ERR(-1);
	return 0;
}

#if 0
template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::riffloop()
{
	for (;;) {
		switch (findck(&mrf, mckRIFF, "RIFF")) {
		  case 0:	return 0;
		  case 1:	break;
		  default:	return AKAMOZ_ERR(-1);
		}
		DWORD dw;
		if (mckRIFF.getdword(dw) < 1)
			return AKAMOZ_ERR(-1);
		if (fcceq(dw, "AVIX")) {
			switch (moviloop()) {
			  case 0:	break;
			  case 1:	return 1;
			  default:	return AKAMOZ_ERR(-1);
			}
		}
	}
	return 0;
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::moviloop()
{
	int r;
	while (!mckRIFF.isckend()) {
		if ((r = mckRIFF.findlist(mckMovi, "movi")) < 0)
			return AKAMOZ_ERR(-1);
		if (r == 0)
			break;
		switch (recloop()) {
		  case 0:	break;
		  case 1:	return 1;
		  default:	return AKAMOZ_ERR(-1);
		}
	}
	if (mckRIFF.cleanup() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::recloop()
{
	int bGot = 0;
	int r;
	while (!mckMovi.isckend()) {
		if ((r = mckRec.setup(&mrf)) < 0)
			return AKAMOZ_ERR(-1);
		if (r == 0)
			break;
		if (mckRec.ckidis("LIST")) {
			DWORD dw;
			mbRecChunk = 1;
			if (mckRec.getdword(dw) < 1)
				return AKAMOZ_ERR(-1);
			if (fcceq(dw, "rec ")) {
				switch (streamloop()) {
				  case 0:	break;
				  case 1:	return 1;
				  default:	return AKAMOZ_ERR(-1);
				}
			}
		}
		else {
			if (checkStreamPrefix(mckRec)) {
				bGot = 1;
				mckcnt[mckRec.ckid()]++;
			}
		}
		mbRecChunk = 0;
		if (mckRec.cleanup() < 0)
			return AKAMOZ_ERR(-1);
		if (bGot)
			return 1;
	}
	if (mckMovi.cleanup() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::streamloop()
{
	int bGot = 0;
	while (!mckRec.isckend()) {
		TChunkReader ck;
		switch (ck.setup(&mrf)) {
		  case 0:
			break;
		  case 1:
			if (checkStreamPrefix(ck)) {
				bGot = 1;
				mckcnt[ck.ckid()]++;
			}
			break;
		  default:
			return AKAMOZ_ERR(-1);
		}
		if (ck.cleanup() < 0)
			return AKAMOZ_ERR(-1);
		if (bGot)
			return 1;
	}
	return 0;
}

template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::getNextFrame()
{
	if (mbRecChunk) {
		switch (streamloop()) {
		  case 0:	break;
		  case 1:	return 1;
		  default:	return AKAMOZ_ERR(-1);
		}
		mbRecChunk = 0;
		if (mckRec.cleanup() < 0)
			return AKAMOZ_ERR(-1);
	}
	switch (recloop()) {
	  case 1:	return 1;
	  case 0:	break;
	  default:	return AKAMOZ_ERR(-1);
	}
	switch (moviloop()) {
	  case 1:	return 1;
	  case 0:	break;
	  default:	return AKAMOZ_ERR(-1);
	}
	switch (riffloop()) {
	  case 1:	return 1;
	  case 0:	break;
	  default:	return AKAMOZ_ERR(-1);
	}
	return 0;
}
#else
template <typename TRF, typename TEND>
int AVISingleStreamBase<TRF, TEND>::getNextFrame(DataBuffer *pdb)
{
	int found = 0;
	DWORD dw;
	int r;

	if (mnStreamIndex < 0)
		return AKAMOZ_ERR(-1);
	if (mbRecChunk)
		goto L2;
	else
		goto L1;

	for (;;) {
		for (;;) {
			if ((r = findck(&mrf, mckRIFF, "RIFF")) < 0)
				return AKAMOZ_ERR(-1);
			if (r == 0)
				return 0;
			if (mckRIFF.getfcc(dw) < 1)
				return AKAMOZ_ERR(-1);
			if (fcceq(dw, "AVIX"))
				break;
			if (mckRIFF.cleanup() < 0)
				return AKAMOZ_ERR(-1);
		}
		while (!mckRIFF.isckend()) {
			if (mckRIFF.findlist(mckMovi, "movi") < 0)
				return AKAMOZ_ERR(-1);
			if (!mckMovi.isReady())
				break;
		L1:
			while (!mckMovi.isckend()) {
				if (mckRec.setup(&mrf) < 1)
					return AKAMOZ_ERR(-1);
				if (mckRec.ckidis("LIST")) {
					if (mckRec.getfcc(dw) < 1)
						return AKAMOZ_ERR(-1);
					if (fcceq(dw, "rec ")) {
						mbRecChunk = 1;
					L2:
						while (!mckRec.isckend()) {
							TChunkReader ck;
							if (ck.setup(&mrf) < 1)
								return AKAMOZ_ERR(-1);
							if (checkStreamPrefix(ck)) {
								if (readdata(pdb, ck) < 0)
									return AKAMOZ_ERR(-1);
								found = 1;
							}
							if (ck.cleanup() < 0)
								return AKAMOZ_ERR(-1);
							if (found)
								return 1;
						}
						mbRecChunk = 0;
					}
				}
				else {
					if (checkStreamPrefix(mckRec)) {
						if (readdata(pdb, mckRec) < 0)
							return AKAMOZ_ERR(-1);
						found = 1;
					}
				}
				if (mckRec.cleanup() < 0)
					return AKAMOZ_ERR(-1);
				if (found)
					return 1;
			}
			if (mckMovi.cleanup() < 0)
				return AKAMOZ_ERR(-1);
		}
		if (mckRIFF.cleanup() < 0)
			return AKAMOZ_ERR(-1);
	}
	return 0;
}
#endif

#if 0
template <typename TRF, typename TEND>
void AVISingleStreamBase<TRF, TEND>::dumpckcnt()
{
	std::map<DWORD, int>::const_iterator i;
	for (i = mckcnt.begin(); i != mckcnt.end(); i++)
		std::cerr << dumpfcc(i->first) << ":" << i->second << " ";
	std::cerr << "\r" << std::flush;
}
#endif

} // namespace akamoz4

#endif // #ifndef __AVISTREAM3_T_INCLUDED
