// $Id: akamoz-base.h 353 2015-09-19 16:06:32Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __AKAMOZ_BASE_INCLUDED
#define __AKAMOZ_BASE_INCLUDED

#include <stdio.h>
#include <string.h>
#include <ostream>
#include <iostream>
#include <iomanip>
#include <stdexcept>
#include <akamoz-environ.h>

#ifndef AKAMOZ_NULL
#define AKAMOZ_NULL	(0)
#endif

#ifndef AKAMOZ_NEW
#define AKAMOZ_NEW(x)	new x
#endif

#ifndef AKAMOZ_DELETE
#define AKAMOZ_DELETE(x)	delete (x)
#endif

#ifndef AKAMOZ_DELARRAY
#define AKAMOZ_DELARRAY(x)	delete [] (x)
#endif

#ifndef AKAMOZ_DEBUGLOG
#ifdef AKAMOZ_USE_DEBUGLOG
#define AKAMOZ_DEBUGLOG(x)	((void)(x))
#else
#define AKAMOZ_DEBUGLOG(x)	((void)(0))
#endif
#endif

// if you want to use these facilities from C,
// redifine these by using printf.
// if you also want to use pdbout facility from C,
// define the function like extern "C" debug_printf,
// and call use akamoz4::printf(std::ostream & ...) with *pdbout.

#ifndef __AKAMOZ_DB
#define __AKAMOZ_DB(os, sz)	(os << sz << "(" << __LINE__ << "): ")
#endif
#ifndef AKAMOZ_DB
#define AKAMOZ_DB(os, sz)	AKAMOZ_DEBUGLOG(__AKAMOZ_DB(os, sz))
#endif

#ifndef __AKAMOZ_REPORT
#define __AKAMOZ_REPORT	(__AKAMOZ_DB(*::akamoz4::pdbout, __FILE__))
#endif
#ifndef AKAMOZ_REPORT
#define AKAMOZ_REPORT	(AKAMOZ_DEBUGLOG(__AKAMOZ_REPORT))
#endif

#ifndef __AKAMOZ_ERR
#define	__AKAMOZ_ERR	(__AKAMOZ_REPORT << "Error\n")
#endif
#ifndef AKAMOZ_ERR
#define AKAMOZ_ERR(x)	(AKAMOZ_DEBUGLOG(__AKAMOZ_ERR), (x))
#endif

#ifndef AKAMOZ_ERRPTR
#define AKAMOZ_ERRPTR(T)	(AKAMOZ_DEBUGLOG(__AKAMOZ_ERR), ((T)0))
#endif

#ifndef AKAMOZ_MARK
#define AKAMOZ_MARK	(AKAMOZ_DEBUGLOG(__AKAMOZ_REPORT << "MARK\n"))
#endif

// this facility cannot use in C,
// because it relies on C++ overload functionality.
// debug_putvalue is used to avoid evaluating x twice.
#ifndef __AKAMOZ_V
#define __AKAMOZ_V	((*::akamoz4::pdbout << #x "="), ::akamoz4::debug_putvalue(x))
#endif
#ifndef AKAMOZ_V
#define AKAMOZ_V(x)	(AKAMOZ_DEBUGLOG(__AKAMOZ_V(x)))
#endif

#ifndef AKAMOZ_OSTRDEC
#define AKAMOZ_OSTRDEC(x) std::setw(x) << std::setfill(' ') << std::dec
#endif

#ifndef AKAMOZ_OSTRDEC0
#define AKAMOZ_OSTRDEC0(x) std::setw(x) << std::setfill('0') << std::dec
#endif

#ifndef AKAMOZ_OSTRHEX0
#define AKAMOZ_OSTRHEX0(x) std::setw(x) << std::setfill('0') << std::hex
#endif

#ifndef AKAMOZ_OSTRFIXED
#ifdef AKAMOZ_NO_IOSFIXED
#define AKAMOZ_OSTRFIXED(w, p) std::setw(w) << std::setfill(' ') << std::setprecision(p)
#else
#define AKAMOZ_OSTRFIXED(w, p) std::fixed << std::setw(w) << std::setfill(' ') << std::setprecision(p)
#endif
#endif

#ifdef AKAMOZ_DEBUG
#define AKAMOZ_ASSUME(x)	do { if (!(x)) return AKAMOZ_ERR(-1); } while (0)
#else
#define AKAMOZ_ASSUME(x)
#endif

#include <new>

// for debug macros
namespace akamoz4 {
extern std::ostream *pdbout;
extern std::ostream nullout;
} // namespace akamoz4

namespace akamoz2 {

typedef unsigned char BYTE;
typedef signed char SBYTE;
typedef char CHAR;
typedef const CHAR CCHAR;
typedef unsigned short USHORT;
typedef unsigned short WORD;
typedef signed short SWORD;
typedef unsigned int UINT;
typedef unsigned long DWORD;

typedef DWORD PTRINT;

#if defined(AKAMOZ_HAVE_LONGLONG)
typedef long long LLONG;
typedef unsigned long long ULLONG;
typedef long long SQWORD;
typedef unsigned long long QWORD;
#elif defined(AKAMOZ_HAVE_INT64)
typedef __int64 LLONG;
typedef unsigned __int64 ULLONG;
typedef __int64 SQWORD;
typedef unsigned __int64 QWORD;
#endif

template <typename T>
class allocator {
  public:
	template <typename TSZ>
	static T *allocate(TSZ n) { return new(std::nothrow) T[n]; }
	static void freeup(T *p) { delete [] p; }
};

template <typename T, class TALLOC = allocator<T> >
class darray {	// akamoz2
  public:
	typedef T TData;
	darray() throw() { _init(); }
	template <typename TSZ>
	darray(TSZ nSize) throw() { _init(); _allocate(nSize); }
	~darray() throw() { _cleanup(); }
	template <typename TSZ>
	int allocate(TSZ nSize) throw() {
		if (mp != NULL)
			return AKAMOZ_ERR(-1);
		_allocate(nSize);
		return mp == NULL ? -1 : 0;
	}
	int giveaway(darray &da) throw() {
		da._cleanup();
		da.mp = mp;
		_cleanup();
		return 0;
	}
	int isReady() const throw() { return mp != NULL; }
	int cleanup() throw() { _cleanup(); _init(); return 0; }
	operator T* () throw() { return mp; }
	operator const T* () const throw() { return mp; }
	template <typename TSZ>
	T &at(TSZ pos) throw() { return mp[pos]; }
	template <typename TSZ>
	T &at(TSZ pos) const throw() { return mp[pos]; }
	template <typename TSZ>
	T &operator [] (TSZ pos) throw() { return mp[pos]; }
	template <typename TSZ>
	T &operator [] (TSZ pos) const throw() { return mp[pos]; }
	template <typename TSZ>
	T *operator + (TSZ n) throw() { return mp + n; }
	template <typename TSZ>
	const T *operator + (TSZ n) const throw() { return mp + n; }
  private:
	TData *mp;
	darray(const darray<TData> &rda) throw();	// no definition.
	// asignment is private, but used by giveaway().
	void operator = (const darray<TData> &rda) throw() {
		mp = rda.mp;
	}
	void _init() throw() { mp = NULL; }
	template <typename TSZ>
	void _allocate(TSZ nSize) throw() {
		mp = TALLOC::allocate(nSize);
	}
	void _cleanup() throw() {
		if (mp != NULL) TALLOC::freeup(mp);
	}
};

template <typename TDATA, typename TSZ = int>
class GenericBuffer {
  public:
	typedef TDATA TData;
	typedef TSZ TSize;
	GenericBuffer() throw() { _init(); }
	int set(TDATA *p, TSZ n) throw() {
		if (set((const TDATA *)p, n) < 0)
			return AKAMOZ_ERR(-1);
		mp = p;
		return 0;
	}
	int set(const TDATA *p, TSZ n) throw() {
		if (n < 0)
			return AKAMOZ_ERR(-1);
		if ((p == NULL) && (n > 0))
			return AKAMOZ_ERR(-1);
		mp = AKAMOZ_NULL;
		mpConst = p;
		mnCapacity = mnSize = n;
		return 0;
	}
	int reuse(TSZ nSize) throw() {
		if (nSize < 0)
			return AKAMOZ_ERR(-1);
		if (nSize <= mnCapacity) {
			mnSize = nSize;
			return 0;
		}
		if (mda.cleanup() < 0)
			return AKAMOZ_ERR(-1);
		if (mda.allocate(nSize) < 0)
			return AKAMOZ_ERR(-1);
		mnSize = mnCapacity = nSize;
		mpConst = mp = mda;
		return 0;
	}
	int allocate(TSZ nSize) throw() {
		if (mda.isReady())
			return AKAMOZ_ERR(-1);
		return reuse(nSize);
	}
	TSZ size() const throw() { return mnSize; }
	TSZ capacity() const throw() { return mnCapacity; }
	TDATA *getmp() throw() { return mp; }
	TDATA &operator [] (TSZ pos) throw() { return mda[pos]; }
	TDATA operator [] (TSZ pos) const throw() { return mda[pos]; }
	TDATA &at(TSZ pos) {
		if (pos >= mnSize)
			throw std::out_of_range("akamoz2::GenericBuffer<>::at()");
		return mda[pos];
	}
	TDATA at(TSZ pos) const {
		if (pos >= mnSize)
			throw std::out_of_range("akamoz2::GenericBuffer<>::at()");
		return mda[pos];
	}
	TDATA *operator + (TSZ pos) throw() {
		return mda + pos;
	}
	const TDATA *getcp() const throw() {
		return mpConst;
	}
	int cleanup() throw() {
		mda.cleanup();
		_init();
		return 0;
	}
	int giveaway(GenericBuffer<TDATA, TSZ> &rgb, int bKeep = 0) throw() {
		rgb.cleanup();
		if (mda.isReady())
			mda.giveaway(rgb.mda);
		rgb.mp = mp;
		rgb.mpConst = mpConst;
		rgb.mnSize = mnSize;
		rgb.mnCapacity = mnCapacity;
		if (!bKeep)
			cleanup();
		return 0;
	}
	int reallocate(TSZ nSize);
  private:
	darray<TDATA> mda;
	TDATA *mp;
	const TDATA *mpConst;
	TSZ mnSize;
	TSZ mnCapacity;
	void _init() {
		mp = AKAMOZ_NULL;
		mpConst = AKAMOZ_NULL;
		mnSize = mnCapacity = 0;
	}
};

// treat DataBuffer specially.
// new and delete must be handled at the function in the library.
// this is needed to build DLL, which can successfully free 
// the memory which is allocated by application initiated setup, 
// or vise versa.

// declare specialized allocator<BYTE>, without definition.
// it is instantiated in akamoz-allocbyte.cpp.
template <>
class allocator<BYTE> {
  public:
	template <typename TSZ>
	static BYTE *allocate(TSZ n);
	static void freeup(BYTE *p);
};

typedef GenericBuffer<BYTE> DataBuffer;

// simplified std::auto_ptr
// destructor of T must not throw any exceptions.
template <typename T>
class tinyptr {
  public:
	explicit tinyptr(T *p = 0) throw() { mp = p; }
	~tinyptr() throw() { _cleanup(); }
	void set(T *p) throw() { _cleanup(); mp = p; }
	T *get() const throw() { return mp; }
	T *release() throw() { T *p = mp; mp = 0; return p; }
	template <typename DERIVED>
	void takeover(tinyptr<DERIVED> &rp) throw() { set(rp.release()); }
	template <typename BASE>
	void giveaway(tinyptr<BASE> &rp) throw() { rp.set(release()); }
	void cleanup() throw(){ _cleanup(); _init(); }
	T &operator * () const throw() { return *mp; }
	T *operator -> () const throw() { return mp; }
	bool isNULL() const throw() { return mp == AKAMOZ_NULL; }
  private:
	T *mp;
	tinyptr(const tinyptr<T> &);
	void operator = (const tinyptr<T> &);
	void _init() throw() { mp = 0; }
	void _cleanup() throw() { if (mp != 0) delete mp; }
};

} // namespace akamoz2

namespace akamoz {

typedef akamoz2::BYTE BYTE;
typedef akamoz2::SBYTE SBYTE;
typedef akamoz2::CHAR CHAR;
typedef akamoz2::CCHAR CCHAR;
typedef akamoz2::USHORT USHORT;
typedef akamoz2::WORD WORD;
typedef akamoz2::SWORD SWORD;
typedef akamoz2::UINT UINT;
typedef akamoz2::DWORD DWORD;

typedef akamoz2::PTRINT PTRINT;

typedef akamoz2::LLONG LLONG;
typedef akamoz2::ULLONG ULLONG;
typedef akamoz2::SQWORD SQWORD;
typedef akamoz2::QWORD QWORD;

template <typename T, typename TSZ = int>
class darray {
  public:
	typedef T TData;
	typedef TSZ TSize;
	darray() { _init(); }
	darray(TSZ nSize) { _init(); _allocate(nSize); }
	int allocate(TSZ nSize) {
		if (mpData != NULL)
			return -1;
		return _allocate(nSize);
	}
	int reuse(TSZ nSize) {
		if ((mpData != NULL) && (nSize < mnCapacity)) {
			mnSize = nSize;
			return 0;
		}
		cleanup();
		return _allocate(nSize);
	}
	int reallocate(TSZ nSize) {
		if (mpData == NULL)
			return allocate(nSize);
		if (nSize < mnCapacity) {
			mnSize = nSize;
			return 0;
		}
		T *pOld = mpData;
		int nOldSize = mnSize;
		if (_allocate(nSize) < 0) {
			mpData = pOld;
			return AKAMOZ_ERR(-1);
		}
		memcpy(mpData, pOld, sizeof(T) * nOldSize);
		AKAMOZ_DELARRAY(pOld);
		return 0;
	}
	int cleanup() { _cleanup(); _init(); return 0; }
	void clear() { memset(mpData, 0, mnSize * sizeof(T)); }
	void set(const T *p, int n) { memcpy(mpData, p, n * sizeof(T)); }
	void setat(int at, const T *p, int n) { memcpy(mpData + at, p, n * sizeof(T)); }
	TSZ size() { return mnSize; }
	TSZ capacity() { return mnCapacity; }
	~darray() { _cleanup(); }
	operator T* () { return mpData; }
	operator const T* () const { return mpData; }
	T* operator + (int n) { return mpData + n; }
	T* operator + (UINT n) { return mpData + n; }
	const T* operator + (int n) const { return mpData + n; }
	const T* operator + (UINT n) const { return mpData + n; }
	T &at(TSZ pos) { return mpData[pos]; }
	const T &at(TSZ pos) const { return mpData[pos]; }
	T &operator [] (TSZ pos) { return mpData[pos]; }
	const T &operator [] (TSZ pos) const { return mpData[pos]; }
  private:
	T *mpData;
	TSZ mnSize;
	TSZ mnCapacity;
	darray(const darray<T, TSZ> &);
	darray<T, TSZ> &operator = (const darray<T, TSZ> &);
	void _init() { mpData = NULL; mnSize = 0; mnCapacity = 0; }
	int _allocate(TSZ nSize) {
		mpData = AKAMOZ_NEW(T[nSize]);
		if (mpData == NULL)
			return -1;
		mnSize = mnCapacity = nSize;
		return 0;
	}
	void _cleanup() { if (mpData != NULL) AKAMOZ_DELARRAY(mpData); }
};

typedef darray<BYTE> DataBuffer;

}

namespace akamoz4 {

typedef akamoz2::BYTE BYTE;
typedef akamoz2::SBYTE SBYTE;
typedef akamoz2::CHAR CHAR;
typedef akamoz2::CCHAR CCHAR;
typedef akamoz2::USHORT USHORT;
typedef akamoz2::WORD WORD;
typedef akamoz2::SWORD SWORD;
typedef akamoz2::UINT UINT;
typedef akamoz2::DWORD DWORD;

typedef akamoz2::PTRINT PTRINT;

#if defined(AKAMOZ_HAVE_LONGLONG)
typedef akamoz2::LLONG LLONG;
typedef akamoz2::ULLONG ULLONG;
typedef akamoz2::SQWORD SQWORD;
typedef akamoz2::QWORD QWORD;
#elif defined(AKAMOZ_HAVE_INT64)
typedef akamoz2::LLONG LLONG;
typedef akamoz2::ULLONG ULLONG;
typedef akamoz2::SQWORD SQWORD;
typedef akamoz2::QWORD QWORD;
#endif

typedef akamoz2::DataBuffer DataBuffer;

template <typename T>
class DefaultDeleter {
  public:
	void operator () (T *p) { delete p; }
};

template <typename T, typename D = DefaultDeleter<T> >
class tiny_ptr {
  public:
	typedef T element_type;
	typedef T *pointer;
	typedef const T *const_pointer;
	typedef D deleter_type;
	tiny_ptr(pointer p = NULL) { mp = p; }
	~tiny_ptr() { if (mp != NULL) reset(); }
	pointer release() { pointer p = mp; mp = NULL; return p; }
	void reset(pointer p = NULL) {
		if (mp != NULL) md(mp);
		mp = p;
	}
	pointer get() { return mp; }
	const_pointer get() const { return mp; }
	operator pointer() { return mp; }
	operator const_pointer () const { return mp; }
	pointer operator -> () { return mp; }
	const_pointer operator -> () const { return mp; }
	template <typename U, typename E>
	void takeover(tiny_ptr<U, E> &rp) throw() { reset(rp.release()); }
	template <typename U, typename E>
	void giveaway(tiny_ptr<U, E> &rp) throw() { rp.reset(release()); }
  private:
	tiny_ptr(const tiny_ptr<T, D> &);
	void operator = (const tiny_ptr<T, D> &);
	template <typename U> void operator [] (U);
	pointer mp;
	deleter_type md;
};

template <typename T>
class rmptr {
};

template <typename T>
class rmptr<T *> {
  public:
	typedef T type;
};

template <typename T>
T &debug_putvalue(T &x) { *pdbout << x << " (T)\n"; return x; }

template <typename T>
const T &debug_putvalue(const T &x) { *pdbout << x << " (const T)\n"; return x; }

class SwitchDebugOutput {
  public:
	SwitchDebugOutput(std::ostream *p) { mp = pdbout; pdbout = p; }
	SwitchDebugOutput(std::ostream &r) { mp = pdbout; pdbout = &r; }
	~SwitchDebugOutput() { pdbout = mp; }
  private:
	std::ostream *mp;
};

} // namespace akamoz4

#endif
