// $Id: akamoz-data.h 353 2015-09-19 16:06:32Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __AKAMOZ_DATA_INCLUDED
#define __AKAMOZ_DATA_INCLUDED

#include "akamoz-base.h"
#include <stdio.h>

#ifdef AKAMOZ_WIN32
#include "akamoz-windows.h"
#endif

namespace akamoz {

// Iȗ\:
//	BasicStream: rewind̂݉\ȃXg[B
//	RegularStream: setposEgetpos\ȃXg[B
//	LargeStreamBase<TSZ>: TCY֌W̌^włXg[B
//	LargeStream: LargeStreamBase<INT64>
//	DataStream: [U[̃}N`ɉāARegularStreamLargeStreamB
//	LargeStream<TSZ>RegularStream̔hNXB
//	RegularStreamBasicStream̔hNXB
// DataStreamnNXgNX́ADataStreamnNXev[g
//	Ŏwł悤ɂāAev[gCX^X
//	RegularStreamLargeStream󂯎NX𐶐B
//	_: CX^XꂽNXԂɌp֌WȂB
//	--- A邢 ---
//	dynamic_castLargeStreamɑΉĂ邩𒲂ׂB
//	--- Ƃ ---
//	KvȊ֐ev[gɂB
//	_: z֐ɂłȂB
//	--- Ƃ ---
//	KvȊ֐ɂăI[o[ChB
//	_: ߂l݈̂قȂꍇ̓I[o[ChłȂB
//
// A64rbgKvȃNX́A
//	ۂɕKv64rbgg킯B
//	Ⴆ΁At@C64rbgL^ĂA
//	ɃV[NȂ΂ȂȂ󋵂lB
//	2GB𒴂铮t@Cł͔rIOȏ󋵁B
//	ƁALargeStreamgȂƃV[NłȂLargeStreamg낤B
//	A2GBȂRegularStreamłvłB
//	At@CɋL^Ă钷31rbgɎ܂ȂꍇA
//	G[񍐂Ȃ΂ȂȂB
//	ALargeStreamRegularStreamł͏ႤB
//	ʂȏKȊ֐ŏāA̓I[o[Chŕ̂
//	@̂悤ɎvBǂ炩ł悢̂ȂRegularStream
//	킯ARegularStreamŎĂ΂悢B
//
//	Ƃ͂Aʓ|ȂƂɂ͕ςȂ̂ŁA
//	RegularStreamG[ȂLargeStreamɕϊ
//	A_v^Ă̂Ȃit͌p֌Wŉ\jB
//	Ⴆ΁ALargeStreamAdapter::setpośA31rbgɎ܂Ă
//	ƂmFĂRegularStream::setposĂԁA悤ȃC[WB

class DataStream {
  public:
	virtual ~DataStream();
	virtual int getbyte() = 0;
};

//--------------------------------------------------------------------
// DataStreamMemory: read data from a memory block.
//--------------------------------------------------------------------
class DataStreamMemory : public DataStream {
  public:
	DataStreamMemory(const BYTE *pb = AKAMOZ_NULL, int n = 0);
	int setbuf(const BYTE *pb, int n);
	virtual int getbyte();
  private:
	const BYTE *mpbBuf;
	int mnLen;
	int mnPos;
	void _init() {
		mpbBuf = AKAMOZ_NULL;
		mnLen = -1;
		mnPos = 0;
	}
};

//--------------------------------------------------------------------
// BitStream: bit-stream, data is read from DataStream.
//--------------------------------------------------------------------
class BitStream {
  public:
	BitStream(DataStream *pdstrm);
	int setstrm(DataStream *pdstrm);
	int cleanup();
	inline int getbit() { return getbits(1); }
	int getbits(int nBits);

	// TVAL must support operator "=", "<<=" and "|=".
	// return 1 if success, 0 if EOF, negative if fail.
	template <typename TVAL> int getbits(TVAL &rval, int nBits) {
		if (mpdstrm == AKAMOZ_NULL)
			return AKAMOZ_ERR(-1);
		if (nBits <= 0)
			return AKAMOZ_ERR(-1);
		if (nBits > (int)sizeof(TVAL))	// must decrese one for signed type...
			return AKAMOZ_ERR(-1);
		if (mnBitsInBuf == 0)
			return 0;	// EOF
		if (_getbits(rval, nBits) < 0)
			return AKAMOZ_ERR(-1);
		return 1;
	}

	// int peek();
  private:
	DataStream *mpdstrm;
	DWORD mdwBuf;
	int mnBitsInBuf;
	inline void _init() {
		mpdstrm = AKAMOZ_NULL;
		mdwBuf = 0;
		mnBitsInBuf = 0;
	}
	void _fill();
	int _getbits(int nBits);

	// recursion part.
	// return negative if error.
	template <typename TVAL> int _getbits(TVAL &rval, int nBits) {
		if (nBits <= 25) {
			int nData = _getbits(nBits);
			if (nData < 0)
				return AKAMOZ_ERR(-1);
			rval = nData;
			return 0;
		}
		int nData = _getbits(25);
		if (nData < 0)
			return AKAMOZ_ERR(-1);
		if (_getbits(rval, nBits - 25) < 0)
			return AKAMOZ_ERR(-1);
		rval <<= 25;
		rval |= nData;
		return 0;
	}
};

//--------------------------------------------------------------------
// D A T A   S T O R A G E S
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// DataStorage: abstruct class for storing byte stream.
//--------------------------------------------------------------------
class DataStorage {
  public:
	virtual ~DataStorage();
	virtual int putbyte(BYTE b) = 0;
	virtual int write(const BYTE *pb, int nLen) = 0;
	virtual int flush() = 0;
	virtual int cleanup() = 0;
	// the following functions are utilities, not virtual.
	int putwordLE(WORD w);
	int putwordBE(WORD w);
	int putdwordLE(DWORD dw);
	int putdwordBE(DWORD dw);
};

//--------------------------------------------------------------------
// DataStorageMemory: storing data to memory.
//--------------------------------------------------------------------

class DataStorageMemory : public DataStorage {
  public:
	DataStorageMemory(darray<BYTE> *pda = AKAMOZ_NULL);
	int setbuf(darray<BYTE> *pda);
	virtual int putbyte(BYTE b);
	virtual int write(const BYTE *pb, int nLen);
	virtual int flush();
	virtual int cleanup();
	inline int size() const { return mnPos; }
  private:
	darray<BYTE> *mpdabBuf;
	int mnPos;
	int mnSize;
	void _init() {
		mnPos = 0;
		mnSize = 0;
		mpdabBuf = AKAMOZ_NULL;
	}
	int _expand(int nSize);
};

//--------------------------------------------------------------------
// DataStorageStdc: storing data to FILE *
//--------------------------------------------------------------------
class DataStorageStdc : public DataStorage {
  public:
	DataStorageStdc(const char *pfn);
	virtual ~DataStorageStdc();
	virtual int putbyte(BYTE b);
	virtual int write(const BYTE *pb, int nLen);
	virtual int flush();
	virtual int cleanup();
  private:
	FILE *mfp;
	void _init() {
		mfp = AKAMOZ_NULL;
	}
	void _cleanup();
};

//--------------------------------------------------------------------
// BitStorage: store bit stream.
//--------------------------------------------------------------------
class BitStorage {
  public:
	BitStorage(DataStorage *pds = AKAMOZ_NULL);
	int setstrg(DataStorage *pds);
	~BitStorage();
	int putbit(bool b);

	template <typename TVAL>
	int putbits(TVAL val, int nBits) {
		if (nBits < 0)
			return AKAMOZ_ERR(-1);
		while (nBits > 24) {
			if (_putbits(val, 24) < 0)
				return AKAMOZ_ERR(-1);
			nBits -= 24;
			val >>= 24;
		}
		if (_putbits(val, nBits) < 0)
			return AKAMOZ_ERR(-1);
		return 0;
	}

	int flush();
	int cleanup();
	inline int bitpos() const { return mnBitPos; }
	inline int bitlen() const { return mnBitLen; }
	int append(const BYTE *pb, int nBits);

  private:
	DataStorage *mpds;
	darray<BYTE> mdabBuf;
	DWORD mdwBuf;
	int mnBitPos;
	int mnBytePos;
	int mnBitLen;
	void _init();
	int _putbits(DWORD dw, int nBits);
	inline int _putbyte(int c);
	inline int _flush();
};

} // namespace akamoz

namespace akamoz2 {

class ReadBlock {
  public:
	virtual ~ReadBlock();
	int isReady() { return mbReady; }
	int isClean() { return mbClean; }
	virtual int cleanup();
	virtual int read(BYTE *pb, int nSize) = 0;
	virtual int rewind() = 0;
  protected:
	ReadBlock() { _init(); }
	UINT mbReady:1;
	UINT mbClean:1;
  private:
	void _init() {
		mbReady = 0;
		mbClean = 1;
	}
};

class ReadDataBase {
  public:
	ReadDataBase() { _init(); }
	virtual ~ReadDataBase();
	int isReady() { return mbReady; }
	int isClean() { return mbClean; }
	inline int getbyte();
	inline int getwordLE();
	inline int getwordBE();
	template <typename T> inline int getdwordLE(T &rdw);
	template <typename T> inline int getdwordBE(T &rdw);
	template <typename TRB> inline int setup(
		tinyptr<TRB> &prb, BYTE *pbBuf = AKAMOZ_NULL, int nBufsize = 4096
	);
	int read(BYTE *pb, int n);
	virtual int rewind();
	virtual int cleanup();
  protected:
	int mnPos;
	int mnAvail;
	ReadBlock *getblock() throw() { return mprb.get(); }
	virtual int _setup(BYTE *pbBuf, int nBufsize);
	virtual int _fill_and_getbyte();
	virtual int _read(BYTE *pb, int n);
  private:
	tinyptr<ReadBlock> mprb;
	DataBuffer mdaBuf;
	UINT mbClean:1;
	UINT mbReady:1;
	void _init() {
		mnPos = 0;
		mnAvail = 0;
		mbReady = 0;
		mbClean = 1;
	}
};

inline int ReadDataBase::getbyte()
{
	if (mnPos < mnAvail)
		return mdaBuf[mnPos++];
	return _fill_and_getbyte();
}

inline int ReadDataBase::getwordLE()
{
	int c = getbyte();
	int d = getbyte();
	if ((c < 0) || (d < 0))
		return AKAMOZ_ERR(-1);
	return c | (d << 8);
}

inline int ReadDataBase::getwordBE()
{
	int c = getbyte();
	int d = getbyte();
	if ((c < 0) || (d < 0))
		return AKAMOZ_ERR(-1);
	return (c << 8) | d;
}

template <typename T>
inline int ReadDataBase::getdwordLE(T &rdw)
{
	int c = getwordLE();
	int d = getwordLE();
	if ((c < 0) || (d < 0))
		return AKAMOZ_ERR(-1);
	rdw = (T)c | ((T)d << 16);
	return 0;
}

template <typename T>
inline int ReadDataBase::getdwordBE(T &rdw)
{
	int c = getwordLE();
	int d = getwordLE();
	if ((c < 0) || (d < 0))
		return AKAMOZ_ERR(-1);
	rdw = ((T)c << 16) | (T)d;
	return 0;
}

template <typename TRB>
inline int ReadDataBase::setup(tinyptr<TRB> &prb, BYTE *pbBuf, int nBufsize)
{
	mbReady = 0;
	if (!isClean())
		return AKAMOZ_ERR(-1);
	mbClean = 0;
	mprb.takeover(prb);
	if (_setup(pbBuf, nBufsize) < 0)
		return AKAMOZ_ERR(-1);
	mbReady = 1;
	return 0;
}

template <typename TSZ>
class ReadBlockSeekableBase : public ReadBlock {
  public:
	typedef TSZ TSize;
	virtual int setpos(TSZ nPos) = 0;
};

template <typename TSZ>
class ReadFileBase : public ReadDataBase {
  public:
	typedef TSZ TSize;
	typedef ReadBlockSeekableBase<TSZ> TReadBlock;
	ReadFileBase() { _init(); }
	virtual TSZ getpos();
	virtual int setpos(TSZ nPos);
	virtual int rewind();
	virtual int forward(TSZ nPos);
	virtual int cleanup();
  protected:
	virtual int _read(BYTE *pb, int n);
  private:
	TSZ mnBufHead;
	void _init() {
		mnBufHead = 0;
	}
};

typedef ReadFileBase<int> ReadFileGeneric;
typedef ReadFileBase<LLONG> ReadLargeFileGeneric;

template <typename TSZ>
class ReadBlockLimitedBase : public ReadBlockSeekableBase<TSZ> {
  public:
	typedef TSZ TSize;
	typedef ReadFileBase<TSZ> TReadFile;
	typedef ReadBlockLimitedBase<TSize> TMyself;
	typedef ReadBlockSeekableBase<TSZ> TBase;
	static TMyself *create(TReadFile *prf, TSize nSize) throw();
	~ReadBlockLimitedBase();
	static void operator delete(void *p);
	int setup(TReadFile *prf, TSize nSize);
	virtual int cleanup();
	virtual int read(BYTE *pb, int nSize);
	virtual int rewind();
	virtual int setpos(TSZ nPos);
  protected:
	ReadBlockLimitedBase(TReadFile *prf, TSize nSize) {
		_init();
		setup(prf, nSize);
	}
  private:
	TReadFile *mprf;
	TSZ mnSize;
	TSZ mnHead;
	TSZ mnPos;
	void _init() {
		mprf = AKAMOZ_NULL;
		mnSize = 0;
		mnHead = 0;
		mnPos = 0;
	}
};

inline DWORD convfccdw(const BYTE *psz)
{
	return psz[0] | (psz[1] << 8) | (psz[2] << 16) | (psz[3] << 24);
}

inline DWORD convfccdw(const CHAR *psz)
{
	return convfccdw((const BYTE *)psz);
}

inline DWORD convfccdw(const SBYTE *psz)
{
	return convfccdw((const BYTE *)psz);
}

inline int cmpfcc(DWORD dwID, const BYTE *psz)
{
	return (int)(dwID - convfccdw(psz));
}

inline int cmpfcc(DWORD dwID, const char *psz)
{
	return cmpfcc(dwID, (const BYTE *)psz);
}

inline int cmpfcc(const char *psz1, const char *psz2)
{
	return cmpfcc(convfccdw((const BYTE *)psz1), psz2);
}

template <typename TSZ>
class ReadIFFBase {
  public:
	typedef TSZ TSize;
	typedef ReadFileBase<TSZ> TReadFile;
	typedef ReadBlockLimitedBase<TSZ> TReadBlock;
	ReadIFFBase() { _init(); }
	TSZ getcksize() const { return mnSize; }
	const char *getckid() const { return (const char *)maucID; }
	int ckidis(const char *s) { return cmpfcc(s, getckid()) == 0; }
	int ckidis(DWORD dw) { return cmpfcc(dw, getckid()) == 0; }
	virtual ~ReadIFFBase();
	virtual int setup(TReadFile *prf);
	virtual int cleanup();
	virtual int nextck();
	virtual int rewind();	// for outer IFF file
	virtual int setpos(TSZ nPos); // for outer IFF file
	virtual int findck(DWORD dwID, int bFromBegin = 0);
	virtual int findlist(DWORD dwID, int bFromBegin = 0);
	int findck(const BYTE *s, int b = 0) { return findck(convfccdw(s), b); }
	int findck(const CHAR *s, int b = 0) { return findck(convfccdw(s), b); }
	int findck(const SBYTE *s, int b = 0) { return findck(convfccdw(s), b); }
	int findlist(const BYTE *s, int b = 0) { return findlist(convfccdw(s), b); }
	int findlist(const CHAR *s, int b = 0) { return findlist(convfccdw(s), b); }
	int findlist(const SBYTE *s, int b = 0) { return findlist(convfccdw(s), b); }
	TReadFile &body() { return mrfBody; }
  private:
	TReadFile *mprf;
	TReadFile mrfBody;
	tinyptr<TReadBlock> mprbBody;
	TSZ mnSize;
	unsigned char maucID[5];
	TSZ mnChunkHead;
	TSZ mnIFFHead;
	UINT mbUsePadding:1;
	void _init() {
		mprf = AKAMOZ_NULL;
		mnSize = -1;
		mnChunkHead = 0;
		mnIFFHead = 0;
		maucID[0] = maucID[1] = maucID[2] = maucID[3] = maucID[4] = 0;
		mbUsePadding = 1;
	}
	int setupChunk();
};

typedef ReadBlockLimitedBase<int> ReadBlockLimited;
typedef ReadBlockLimitedBase<LLONG> ReadLargeBlockLimited;
typedef ReadIFFBase<int> ReadIFF;
typedef ReadIFFBase<LLONG> ReadLargeIFF;

#ifdef AKAMOZ_HAVE_WIN32FILEAPI

inline HANDLE createFile(
	LPCSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode,
	LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDisposition,
	DWORD dwFlagsAndAttributes, HANDLE hTemplateFile
) {
	return ::CreateFileA(
		lpFileName, dwDesiredAccess, dwShareMode, lpSecurityAttributes,
		dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile
	);
}

inline HANDLE createFile(
	LPCWSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode,
	LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDisposition,
	DWORD dwFlagsAndAttributes, HANDLE hTemplateFile
) {
	return ::CreateFileW(
		lpFileName, dwDesiredAccess, dwShareMode, lpSecurityAttributes,
		dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile
	);
}

class ReadLargeBlockWin32 : public ReadBlockSeekableBase<LLONG> {
  public:
	virtual ~ReadLargeBlockWin32();
	static ReadLargeBlockWin32 *create();
	static void operator delete(void *p);
	template <typename TC>
	int setup(const TC *lpszName);
	virtual int read(BYTE *pb, int nSize);
	virtual int rewind();
	virtual int cleanup();
	virtual int setpos(LLONG nPos);
  protected:
	ReadLargeBlockWin32() { _init(); }
  private:
	HANDLE mhFile;
	void _init() {
		mhFile = INVALID_HANDLE_VALUE;
		mbReady = 0;
	}
};

class ReadLargeFileWin32 : public ReadLargeFileGeneric {
  public:
	template <typename TC>
	int setup(const TC *pszfn);
};

#define AKAMOZ_HAVE_READLARGEFILE
typedef ReadLargeFileWin32 ReadLargeFile;

#endif // AKAMOZ_HAVE_WIN32FILEAPI

} // namespace akamoz2

namespace akamoz4 {

class LittleEndian {
  public:
	template <typename T, typename U>
	static void toword(const T *p, U &w) {
		w = (BYTE)p[0] | ((BYTE)p[1] << 8);
	}
	template <typename T, typename U>
	static void fromword(T *p, U w) {
		p[0] = w & 0xff;
		p[1] = (w >> 8) & 0xff;
	}
	template <typename T, typename U>
	static void todword(const T *p, U &dw) {
		dw = (BYTE)p[0] | ((BYTE)p[1] << 8) |
			((BYTE)p[2] << 16) | ((BYTE)p[3] << 24);
	}
	template <typename T, typename U>
	static void fromdword(T *p, U dw) {
		p[0] = dw & 0xff;
		p[1] = (dw >> 8) & 0xff;
		p[2] = (dw >> 16) & 0xff;
		p[3] = (dw >> 24) & 0xff;
	}
	template <typename T>
	static void tofcc(const T *p, DWORD &dw) {
		todword(p, dw);
	}
	template <typename T>
	static void fromfcc(T *p, DWORD dw) {
		fromdword(p, dw);
	}
};

template <typename TRF, typename ENDIAN = LittleEndian>
class ChunkReader {
  public:
	typedef TRF TReadFile;
	typedef typename TRF::TSize TSize;
	typedef ENDIAN TEndian;
	typedef ChunkReader<TRF, ENDIAN> TThis;
	ChunkReader(TReadFile *prf = NULL, int bUsePad = 1) {
		_init();
		if (prf != NULL)
			setup(prf, bUsePad);
	}
	~ChunkReader() { _cleanup(); }
	template <typename T>
	int getdword(T &dw) {	// 1: got, 0: EOF, -1: ERR
		BYTE buf[4];
		int nRead = mprf->read(buf, 4);
		if (nRead == 0) return 0;
		if (nRead != 4) return AKAMOZ_ERR(-1);
		TEndian::todword(buf, dw);
		return 1;
	}
	template <typename T>
	int getfcc(T &dw) {	// 1: got, 0: EOF, -1: ERR
		BYTE buf[4];
		int nRead = mprf->read(buf, 4);
		if (nRead == 0) return 0;
		if (nRead != 4) return AKAMOZ_ERR(-1);
		TEndian::tofcc(buf, dw);
		return 1;
	}
	int setup(TReadFile *prf, int bUsePad = 1) {	// 1: OK, 0: EOF, -1: ERR
		if (prf == NULL) return AKAMOZ_ERR(-1);
		if (mpos >= 0) return AKAMOZ_ERR(-1);
		mprf = prf;
		mbUsePad = bUsePad;
		int nRet = getfcc(mckid);
		if (nRet < 0) return AKAMOZ_ERR(-1);
		if (nRet == 0) return 0;
		if (getdword(mcklen) < 1) return AKAMOZ_ERR(-1);
		mpos = mprf->getpos();
		if (mpos < 0) return AKAMOZ_ERR(-1);
		return 1;
	}
	int cleanup() {	// 0: OK, -1: ERR
		if (_cleanup() < 0) return AKAMOZ_ERR(-1);
		_init();
		return 0;
	}
	int isReady() const {	// 1: OK, 0: not ready
		return (mpos >= 0) && (mprf != NULL);
	}
	DWORD ckid() const { return mckid; }
	int ckidis(DWORD dw) const { return dw == mckid; }
	template <typename T>
	static DWORD tofcc(const T *s) {
		DWORD dw;
		TEndian::tofcc(s, dw);
		return dw;
	}
	template <typename T>
	bool ckidis(const T *s) const { return tofcc(s) == mckid; }
	int isckend() {	// 0: not eock, 1: EOCK, 2: on pad, -1: ERR
		if (!isReady()) return AKAMOZ_ERR(-1);
		TSize pos = mprf->getpos();
		if (pos < 0) return AKAMOZ_ERR(-1);
		if (pos == mpos + mcklen) return 1;
		if (mbUsePad && (pos == mpos + mcklen + 1)) return 2;
		if (pos > mpos + mcklen) return AKAMOZ_ERR(-1);
		return 0;
	}
	int findck(TThis &subck, DWORD id) {	// 1: found, 0: EOCK, -1: ERR
		if (!isReady()) return AKAMOZ_ERR(-1);
		while (!isckend()) {
			if (subck.setup(mprf) < 1)
				return AKAMOZ_ERR(-1);
			if (subck.ckidis(id))
				return 1;
			if (subck.cleanup() < 0)
				return AKAMOZ_ERR(-1);
		}
		return 0;
	}
	template <typename T>
	int findck(TThis &subck, const T *id) {
		return findck(subck, tofcc(id));
	}
	// 1: found, 0: EOCK, -1: ERROR
	int findlist(TThis &subck, DWORD id, DWORD listid = tofcc("LIST")) {
		if (!isReady()) return AKAMOZ_ERR(-1);
		while (!isckend()) {
			int nFound = findck(subck, listid);
			if (nFound < 0) return AKAMOZ_ERR(-1);
			if (nFound == 0) return 0;
			if (nFound) {
				DWORD lid;
				if (getfcc(lid) < 1) return AKAMOZ_ERR(-1);
				if (lid == id) return 1;
			}
			if (subck.cleanup() < 0)
				return AKAMOZ_ERR(-1);
		}
		return 0;
	}
	template <typename T, typename S>
	int findlist(TThis &subck, const T *id, const S *listid) {
		return findlist(subck, tofcc(id), tofcc(listid));
	}
	template <typename T>
	int findlist(TThis &subck, const T *id) {
		return findlist(subck, tofcc(id));
	}
	TReadFile * operator -> () { return mprf; }
	const TReadFile * operator -> () const { return mprf; }
	TReadFile & operator * () { return *mprf; }
	const TReadFile & operator * () const { return *mprf; }
	TSize getcksize() const {
		if (!isReady()) return AKAMOZ_ERR(-1);
		return mcklen;
	}
  private:
	TReadFile *mprf;
	TSize mpos;
	DWORD mckid;
	DWORD mcklen;
	UINT mbUsePad:1;
	void _init() {
		mprf = NULL;
		mpos = -1;
		mckid = 0;
		mcklen = 0;
		mbUsePad = 1;
	}
	int _cleanup() {
		if (mpos < 0) return 0;
		TSize pos = mprf->getpos();
		if (pos < 0) return AKAMOZ_ERR(-1);
		if (pos > mpos + mcklen) return AKAMOZ_ERR(-1);	// overrun
		mpos += mcklen;
		if (mbUsePad && (mcklen & 1)) mpos++;
		if (mprf->setpos(mpos) < 0) return AKAMOZ_ERR(-1);
		return 0;
	}
};

template <typename TRF, typename ENDIAN>
int findck(TRF *prf, ChunkReader<TRF, ENDIAN> &subck, DWORD id) {
	for (;;) {
		int nRet = subck.setup(prf);
		if (nRet < 0) return AKAMOZ_ERR(-1);
		if (nRet == 0) break;
		if (subck.ckidis(id)) return 1;
		if (subck.cleanup() < 0) return AKAMOZ_ERR(-1);
	}
	return 0;
}

template <typename TRF, typename TC, typename ENDIAN>
int findck(TRF *prf, ChunkReader<TRF, ENDIAN> &subck, const TC *id)
{
	DWORD dw;
	ENDIAN::tofcc(id, dw);
	return findck(prf, subck, dw);
}

template <typename ENDIAN, typename TC>
DWORD tofcc(const TC *id)
{
	DWORD dw;
	ENDIAN::tofcc(id, dw);
	return dw;
}

template <typename TC>
DWORD tofcc(const TC *id)
{
	return tofcc<LittleEndian>(id);
}


template <typename ENDIAN, typename TC>
bool fcceq(DWORD dw, const TC *id)
{
	DWORD dwid;
	ENDIAN::tofcc(id, dwid);
	return dw == dwid;
}

template <typename TC>
bool fcceq(DWORD dw, const TC *id)
{
	return fcceq<LittleEndian>(dw, id);
}

#ifdef AKAMOZ_HAVE_WIN32FILEAPI

#endif

} // namespace akamoz4

#endif // #ifndef __AKAMOZ_DATA_INCLUDED
