// $Id: akamoz-iomanip.h 353 2015-09-19 16:06:32Z you $
/*
 * Copyright (c) 2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __AKAMOZ_IOMANIP_H_INCLUDED
#define __AKAMOZ_IOMANIP_H_INCLUDED

#include <iostream>
#include <akamoz-base.h>

namespace akamoz4 {

// save format flags of ostream
template <typename T>
class SaveOutputStreamFlag {
  public:
	SaveOutputStreamFlag(T &os) : osSave(os) {
		fFlags = os.flags();
		cFill = os.fill();
		szPrec = os.precision();
	}
	~SaveOutputStreamFlag() {
		osSave.flags(fFlags);
		osSave.fill(cFill);
		osSave.precision(szPrec);
	}
  private:
	T &osSave;
	typename T::fmtflags fFlags;
	typename T::char_type cFill;
	typename std::streamsize szPrec;
};

template <typename TM>
class outputmanip_singleparam : public TM {
  public:
	typedef typename TM::TParam TParam;
	TParam param;
	outputmanip_singleparam(TParam p) { param = p; }
};

template <typename TS, typename TM>
TS &operator << (TS &os, const outputmanip_singleparam<TM> &manip)
{
	manip.doit(os, manip.param);
	return os;
}

#define MANIPCLASS(manipname)	outputmanip_ ## manipname

#define DEFINE_OUTPUTMANIP_1PARAM(manipname, paramtype) \
class MANIPCLASS(manipname) { \
  public: \
	typedef paramtype TParam; \
	TParam param; \
	template <typename TS> \
	void doit(TS &, TParam) const; \
}; \
typedef outputmanip_singleparam<MANIPCLASS(manipname)> manipname;

DEFINE_OUTPUTMANIP_1PARAM(sethex0, DWORD)
template <typename TS>
void MANIPCLASS(sethex0)::doit(TS &os, TParam n) const
{
	os << std::hex << std::setw(n) << std::setfill(os.widen('0'));
}

template <typename T>
void printsafechar(T &os, int c)
{
	c &= 0xff;
	if ((c < 0x20) || (c >= 0x7f))
		os << "\\x" << sethex0(2) << c;
	else
		os << (char)c;
}

DEFINE_OUTPUTMANIP_1PARAM(dumpfcc, DWORD)
template <typename TS>
void MANIPCLASS(dumpfcc)::doit(TS &os, TParam dw) const
{
	SaveOutputStreamFlag<TS> sf(os);
	printsafechar(os, dw);
	printsafechar(os, dw >> 8);
	printsafechar(os, dw >> 16);
	printsafechar(os, dw >> 24);
}

} // namespace akamoz4

#endif // #ifndef __AKAMOZ_IOMANIP_H_INCLUDED
