// $Id: akamoz-readfile.t 359 2015-09-20 10:00:28Z you $
/*
 * Copyright (c) 2008-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __AKAMOZ_READFILE_T_INCLUDED
#define __AKAMOZ_READFILE_T_INCLUDED

#include <assert.h>
#include <akamoz-data.h>

namespace akamoz2 {

template <typename TSZ>
int ReadFileBase<TSZ>::cleanup()
{
	int nRet = 0;
	if (ReadDataBase::cleanup() < 0)
		nRet = AKAMOZ_ERR(-1);
	_init();
	return nRet;
}

template <typename TSZ>
int ReadFileBase<TSZ>::_read(BYTE *pb, int nLen)
{
	assert(pb != NULL);
	assert(nLen >= 0);
	assert(mnAvail >= 0);
	mnBufHead += mnAvail;
	ReadBlock *prb = getblock();
	assert(prb != NULL);
	return prb->read(pb, nLen);
}

template <typename TSZ>
TSZ ReadFileBase<TSZ>::getpos()
{
	if (!isReady())
		return AKAMOZ_ERR(-1);
	return mnBufHead + mnPos;
}

template <typename TSZ>
int ReadFileBase<TSZ>::setpos(TSZ nPos)
{
	if (nPos < 0)
		return AKAMOZ_ERR(-1);
	if (!isReady())
		return AKAMOZ_ERR(-1);
	TReadBlock *prb = dynamic_cast<TReadBlock *>(getblock());
	assert(prb != NULL);
	if (prb->setpos(nPos) < 0)
		return AKAMOZ_ERR(-1);
	mnBufHead = nPos;
	mnAvail = mnPos = 0;
	return 0;
}

template <typename TSZ>
int ReadFileBase<TSZ>::rewind()
{
	if (!isReady())
		return AKAMOZ_ERR(-1);
	if (ReadDataBase::rewind() < 0)
		return AKAMOZ_ERR(-1);
	mnBufHead = 0;
	return 0;
}

template <typename TSZ>
int ReadFileBase<TSZ>::forward(TSZ nPos)
{
	if (!isReady())
		return AKAMOZ_ERR(-1);
	TSZ nCur = getpos();
	if (nCur < 0)
		return AKAMOZ_ERR(-1);
	nPos += nCur;
	if (nPos < 0)
		return AKAMOZ_ERR(-1);
	return setpos(nPos);
}

} // namespace akamoz2

#endif // ifndef __AKAMOZ_READFILE_T_INCLUDED
