// $Id: akamoz-readiff.t 359 2015-09-20 10:00:28Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __AKAMOZ_IFF_T_INCLUDED
#define __AKAMOZ_IFF_T_INCLUDED

#include <assert.h>
#include <akamoz-data.h>
#include <new>

namespace akamoz2 {

// RXgN^protectedȂ̂ŁAKcreateĂԕKvB
template <typename TSZ>
ReadBlockLimitedBase<TSZ> *ReadBlockLimitedBase<TSZ>::create(
	TReadFile *prf, TSize nSize
) throw()
{
	return new(std::nothrow) TMyself(prf, nSize);
}

// RXgN^̓CCsetupĂԂ悤ɂȂĂB

template <typename TSZ>
int ReadBlockLimitedBase<TSZ>::setup(TReadFile *prf, TSize nSize)
{
	if (!this->isClean())
		return AKAMOZ_ERR(-1);
	this->mbClean = 0;
	if (prf == AKAMOZ_NULL)
		return AKAMOZ_ERR(-1);
	if (nSize < 0)
		return AKAMOZ_ERR(-1);
	if (!prf->isReady())
		return AKAMOZ_ERR(-1);
	mnHead = prf->getpos();
	if (mnHead < 0)
		return AKAMOZ_ERR(-1);
	mprf = prf;
	mnSize = nSize;
	mnPos = 0;
	this->mbReady = 1;
	return 0;
}

template <typename TSZ>
ReadBlockLimitedBase<TSZ>::~ReadBlockLimitedBase()
{
	cleanup();
}

template <typename TSZ>
int ReadBlockLimitedBase<TSZ>::cleanup()
{
	_init();
	return 0;
}

// newƓt@Cdeleteu߂deleteZq֐B
template <typename TSZ>
void ReadBlockLimitedBase<TSZ>::operator delete(void *p)
{
	::operator delete(p);
}

template <typename TSZ>
int ReadBlockLimitedBase<TSZ>::read(BYTE *pb, int nSize)
{
	if (pb == AKAMOZ_NULL)
		return AKAMOZ_ERR(-1);
	if (mnPos + nSize > mnSize)
		nSize = mnSize - mnPos;
	if (nSize < 0)
		return AKAMOZ_ERR(-1);
	if (!this->isReady())
		return AKAMOZ_ERR(-1);
	if (nSize == 0)
		return 0;
	assert(mprf != AKAMOZ_NULL);
	int nRead = mprf->read(pb, nSize);
	if (nRead < 0)
		return AKAMOZ_ERR(-1);
	mnPos += nRead;
	return nRead;
}

template <typename TSZ>
int ReadBlockLimitedBase<TSZ>::rewind()
{
	if (!this->isReady())
		return AKAMOZ_ERR(-1);
	assert(mprf != AKAMOZ_NULL);
	if (mprf->setpos(mnHead) < 0)
		return AKAMOZ_ERR(-1);
	mnPos = 0;
	return 0;
}

template <typename TSZ>
int ReadBlockLimitedBase<TSZ>::setpos(TSZ nPos)
{
	if (!this->isReady())
		return AKAMOZ_ERR(-1);
	if (nPos < 0)
		return AKAMOZ_ERR(-1);
	if (nPos > mnSize)
		return AKAMOZ_ERR(-1);
	assert(mprf != AKAMOZ_NULL);
	if (mprf->setpos(mnHead + nPos) < 0)
		return AKAMOZ_ERR(-1);
	mnPos = nPos;
	return 0;
}

template <typename TSZ>
ReadIFFBase<TSZ>::~ReadIFFBase()
{
	cleanup();
}

template <typename TSZ>
int ReadIFFBase<TSZ>::setupChunk()
{
	assert(mprf != NULL);
	maucID[0] = maucID[1] = maucID[2] = maucID[3] = maucID[4] = 0;
	mnSize = -1;
	mnChunkHead = mprf->getpos();
	if (mnChunkHead < 0)
		return AKAMOZ_ERR(-1);
	int c = mprf->getbyte();
	if (c < 0)
		return AKAMOZ_ERR(-1);
	maucID[0] = c;
	c = mprf->getbyte();
	if (c < 0)
		return AKAMOZ_ERR(-1);
	maucID[1] = c;
	c = mprf->getbyte();
	if (c < 0)
		return AKAMOZ_ERR(-1);
	maucID[2] = c;
	c = mprf->getbyte();
	if (c < 0)
		return AKAMOZ_ERR(-1);
	maucID[3] = c;
	maucID[4] = 0;
	if (mprf->getdwordLE(mnSize) < 0)
		return AKAMOZ_ERR(-1);

	if (mrfBody.cleanup() < 0)
		return AKAMOZ_ERR(-1);
	// mprbBody is destroied by mrfBody.
	mprbBody.set(TReadBlock::create(mprf, getcksize()));
	if (mprbBody.isNULL())
		return AKAMOZ_ERR(-1);
	if (mrfBody.setup(mprbBody) < 0)
		return AKAMOZ_ERR(-1);

	return 0;
}

template <typename TSZ>
int ReadIFFBase<TSZ>::setup(TReadFile *prf)
{
	if (mprf != NULL)
		return AKAMOZ_ERR(-1);	// not clean
	if (prf == NULL)
		return AKAMOZ_ERR(-1);	// invalid parameter
	mprf = prf;
	mnIFFHead = mprf->getpos();
	if (mnIFFHead < 0)
		return AKAMOZ_ERR(-1);
	if (setupChunk() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template <typename TSZ>
int ReadIFFBase<TSZ>::cleanup()
{
	int nRet = 0;
	if (mrfBody.cleanup() < 0)
		nRet = AKAMOZ_ERR(-1);
//	if ((mprf != NULL) && (mprf->cleanup() < 0))
//		nRet = AKAMOZ_ERR(-1);
	_init();
	return nRet;
}

template <typename TSZ>
int ReadIFFBase<TSZ>::nextck()
{
	if (mprf == NULL)
		return AKAMOZ_ERR(-1);
	int pad = mbUsePadding ? mnSize & 1 : 0;
	if (mprf->setpos(mnChunkHead + 8 + mnSize + pad) < 0)
		return AKAMOZ_ERR(-1);
	if (setupChunk() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template <typename TSZ>
int ReadIFFBase<TSZ>::rewind()
{
	if (mprf == NULL)
		return AKAMOZ_ERR(-1);
	if (mprf->setpos(mnIFFHead) < 0)
		return AKAMOZ_ERR(-1);
	if (setupChunk() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template <typename TSZ>
int ReadIFFBase<TSZ>::setpos(TSZ nPos)
{
	if (mprf == NULL)
		return AKAMOZ_ERR(-1);
	if (mprf->setpos(nPos) , 0)
		return AKAMOZ_ERR(-1);
	if (setupChunk() < 0)
		return AKAMOZ_ERR(-1);
	return 0;
}

template <typename TSZ>
int ReadIFFBase<TSZ>::findck(DWORD dwID, int bFromBegin)
{
	if (mprf == NULL)
		return AKAMOZ_ERR(-1);
	if (bFromBegin) {
		if (rewind() < 0)
			return AKAMOZ_ERR(-1);
	}
	for (;;) {
		if (ckidis(dwID))
			break;
		if (nextck() < 0)
			return AKAMOZ_ERR(-1);
	}
	return 0;
}

template <typename TSZ>
int ReadIFFBase<TSZ>::findlist(DWORD dwID, int bFromBegin)
{
	if (mprf == NULL)
		return AKAMOZ_ERR(-1);
	if (bFromBegin) {
		if (rewind() < 0)
			return AKAMOZ_ERR(-1);
	}
	for (;;) {
		if (ckidis("LIST")) {
			DWORD dw;
			if (body().getdwordLE(dw) < 0)
				return AKAMOZ_ERR(-1);
			if (dwID == dw)
				break;
		}
		if (nextck() < 0)
			return AKAMOZ_ERR(-1);
	}
	return 0;
}

} // namespace akamoz2

#endif // ifndef __AKAMOZ_IFF_T_INCLUDED
