// $Id: akamoz-util.h 353 2015-09-19 16:06:32Z you $
/*
 * Copyright (c) 2007-2015 akamoz.jp
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __AKAMOZ_UTIL_INCLUDED
#define __AKAMOZ_UTIL_INCLUDED

#include <akamoz-base.h>

#ifdef AKAMOZ_POSIX
#include <unistd.h>
#include <termios.h>
#endif

namespace akamoz {

class ParseOption {	// akamoz::
  public:
	ParseOption(CCHAR *pcSwitch, CCHAR *pcOption);
	virtual ~ParseOption();
	int parse(int argc, CHAR * const argv[]);
  protected:
	virtual int setopt(int c, CCHAR *param) = 0;
  private:
	ParseOption(const ParseOption &);
	ParseOption &operator = (const ParseOption &);
	CCHAR *mpcSwitch;
	CCHAR *mpcOption;
};

};

namespace akamoz2 {

struct ParseOptionInfo {
	CCHAR *pszOpt;
	UINT bHaveValue:1;
	int nID;
	const void *pcvUserData;
	void *pvUserData;
};

template <typename T>
CCHAR *getuint(CCHAR *s, T &rn)
{
	if (s == NULL) return NULL;
	if (!isdigit(*s)) return NULL;
	rn = 0;
	while (isdigit(*s)) {
		T n = rn;
		rn = rn * 10 + *s++ - '0';
		if (rn / 10 - n != 0)
			return NULL;
	}
	return s;
}

template <typename TC>
const TC *checkdelim(const TC *s, TC c)
{
	if (s == NULL)
		return NULL;
	if (*s != c)
		return NULL;
	return s + 1;
}

class ParseOption {	// akamoz2::
  public:
	ParseOption();
	virtual int parse(int argc, CCHAR * const argv[], const ParseOptionInfo *pinfo);
	CCHAR *getarg();
	void usage(std::ostream &os, const ParseOptionInfo *pinfo);
  protected:
	virtual int proc(const ParseOptionInfo *pinfo, CCHAR *parg) = 0;
	void setarg(CCHAR *p) { mpNextArg = p; }
  private:
	int mnArgc;
	int mnCnt;
	CCHAR * const *mpArgv;
	CCHAR *mpNextArg;
	CCHAR *compare(CCHAR *s, CCHAR *t);
};

#ifdef AKAMOZ_POSIX
class PosixTerminalMode {
  public:
	PosixTerminalMode() {
		curavail = 0;
		init();
	}
	~PosixTerminalMode() {
		_restore();
	}
	int init() {
		oldavail = 0;
		if (tcgetattr(0, &oldmode) != 0)
			return AKAMOZ_ERR(-1);
		oldavail = 1;
		_setcur(oldmode);
		return 0;
	}
	int update() {
		curavail = 0;
		if (tcgetattr(0, &curmode) != 0)
			return AKAMOZ_ERR(-1);
		curavail = 1;
		return 0;
	}
	int restore() {
		if (_restore() < 0)
			return AKAMOZ_ERR(-1);
		_setcur(oldmode);
	}
	int apply() {
		if (!curavail)
			return AKAMOZ_ERR(-1);
		if (_set(curmode) < 0)
			return AKAMOZ_ERR(-1);
		return 0;
	}
	int getcur(struct termios &mode) const {
		if (!curavail)
			return AKAMOZ_ERR(-1);
		mode = curmode;
		return 0;
	}
	int getorig(struct termios &mode) const {
		if (!oldavail)
			return AKAMOZ_ERR(-1);
		mode = oldmode;
		return 0;
	}
	int set(const struct termios &mode) {
		if (_set(mode) < 0)
			return AKAMOZ_ERR(-1);
		_setcur(mode);
		return 0;
	}
	int setlflag(tcflag_t flag) {
		if (!curavail)
			return AKAMOZ_ERR(-1);
		curmode.c_lflag |= flag;
		return 0;
	}
	int resetlflag(tcflag_t flag) {
		if (!curavail)
			return AKAMOZ_ERR(-1);
		curmode.c_lflag &= ~flag;
		return 0;
	}
  private:
	UINT oldavail:1;
	UINT curavail:1;
	struct termios oldmode;
	struct termios curmode;
	int _set(const struct termios &mode) {
		if (tcsetattr(0, TCSANOW, &mode) != 0)
			return AKAMOZ_ERR(-1);
		return 0;
	}
	int _restore() {
		if (!oldavail)
			return AKAMOZ_ERR(-1);
		if (_set(oldmode) < 0)
			return AKAMOZ_ERR(-1);
		return 0;
	}
	void _setcur(const struct termios &cur) {
		curmode = cur;
		curavail = 1;
	}
};
#endif

} // akamoz2

namespace akamoz4 {

std::ostream &printf(std::ostream &os, const char *fmt ...);

} // namespace akamoz4

#endif
