# $Id: stdgnu.mk 360 2015-09-20 10:00:47Z you $
# Copyright (c) 2008-2015 akamoz.jp
# All rights reserved.
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

ifndef SRCPATH
$(error SRCPATH is not defined.)
endif

OBJ ?= .o
DSTPATH ?= .
RES ?= .coff

CPPFLAGS += $(addprefix -D,$(CPPMACROS))
CFLAGS += $(addprefix -D,$(CMACROS))
CXXFLAGS += $(addprefix -D,$(CXXMACROS))

CXX = g++
CPPFLAGS += -Werror -W -Wall -I $(AKAMOZ_ROOT)/include -mtune=pentium-m
COMPILE_CXX ?= $(CXX) -c $(CPPFLAGS) $(CXXFLAGS) $< -o $@

RCRES ?= windres
RCFLAGS += -I $(SRCPATH)
COMPILE_RC ?= $(RCRES) $(RCFLAGS) -o $@ $<

LD = g++
LDFLAGS += $(foreach path,$(LDPATH),-L$(path))
ifdef AKAMOZ_LIBPATH
LDFLAGS += -L$(AKAMOZ_LIBPATH)
endif
EXPAND_LIBNAME ?= $(foreach lib, $(LIBS), -l$(lib))
LINK_OBJS ?= $(LD) $(LDFLAGS) -o $@ $^ $(LOADLIBES) $(LDLIBS)

LIBNAME = lib$(BASENAME).a
MAKE_LIB ?= rm -f $@ && $(AR) -scr $@ $^

ifeq ("$(BLDTYPE)", "debug")
CPPFLAGS += -g
LDFLAGS += -g
endif

ifeq ("$(BLDTYPE)", "release")
CPPFLAGS += -O2
endif

ifeq ("$(BLDTYPE)", "")
CPPFLAGS += -O
endif

%$(OBJ): $(SRCPATH)/%.cpp
	$(COMPILE_CXX)

%$(RES): $(SRCPATH)/%.rc
	$(COMPILE_RC)

ALLOBJS = $(CPPSRCS:.cpp=$(OBJ))

MKDEP = g++
MAKE_DEPEND = $(MKDEP) -MM -MT $${SRC%$(SRCEXT)}$(OBJEXT) $(CPPFLAGS) $(SRCPATH)/$${SRC} >> depend.mk

include $(AKAMOZ_ROOT)/mk/akamoz-macros.mk
include $(SRCPATH)/build.mk
-include depend.mk

depend: depend-remove depend-cpp depend-rc

depend-remove:
	rm -f depend.mk

OBJEXT ?= $(OBJ)

depend-cpp: SRCEXT=.cpp
depend-cpp:
	for SRC in $(CPPSRCS); do $(MAKE_DEPEND) || exit 1; done

depend-rc: SRCEXT=.rc
depend-rc: OBJEXT=$(RES)
depend-rc: CPPFLAGS += -x c
depend-rc:
	for SRC in $(RCSRCS); do $(MAKE_DEPEND) || exit 1; done

clean:
	rm -f *$(OBJ)
	rm -f *.stackdump
